#include "StdAfx.h"
#include "Channel3.h"
#include <stdio.h>

CChannel3::CChannel3()
{
	// Kommunikationsbasis 
	m_channelBase = NULL;

	// Die Umgebung des verwendeten CRC32 initialisieren
	InitCRC32Table();

	// Umgebung zuruecksetzen
	Reset();

	// Callbacks intialisieren
	m_callBack1 = NULL;
	m_callBack2 = NULL;
}

CChannel3::~CChannel3()
{
}

void CChannel3::Reset()
{
	m_result = IV_ERROR_SUCCESS;
	ResetError();
	memset(m_data, 0, sizeof(m_data));
}

//#########################################################
//######################### CRC ###########################
//#########################################################

// Reflection is a requirement for the official CRC-32 standard.
// You can create CRCs without it, but they won't conform to the standard.
unsigned int CChannel3::Reflect(unsigned int ref, char ch)
{// Used only by InitCRC32Table()

	unsigned int value = 0;

	// Swap bit 0 for bit 7
	// bit 1 for bit 6, etc.
	int i;
	for(i=1; i < (ch + 1); i++)
	{
		if(ref & 1)
			value |= 1 << (ch - i);
		ref >>= 1;
	}

	return value;
}

// Call this function only once to initialize the CRC table.
void CChannel3::InitCRC32Table() 
{// Called by Init

	// This is the official polynomial used by CRC-32 
	// in PKZip, WinZip and Ethernet. 
	unsigned int ulPolynomial = 0x04c11db7;

	// 256 values representing ASCII character codes.
	int i, j;
	for(i=0; i <= 0xFF; i++)
	{
		m_crc32_table[i] = Reflect(i, 8) << 24;
		for (j=0; j < 8; j++) {

			m_crc32_table[i] = (m_crc32_table[i] << 1) ^ (m_crc32_table[i] & (1 << 31) ? ulPolynomial : 0);
		}

		m_crc32_table[i] = Reflect(m_crc32_table[i], 32);
	}
}

// This function uses the crc32_table lookup table
// to generate a CRC for ucData
unsigned int CChannel3::GetCRC32(unsigned char *data, unsigned int size, unsigned int crc, unsigned char closure)
{
	// Be sure to use unsigned variables,
	// because negative values introduce high bits
	// where zero bits are required.
	//unsigned int crc = 0xffffffff;
	int len;
	unsigned char *buffer;

	len = size;
	// Save the text in the buffer.
	buffer = data;
	// Perform the algorithm on each character
	// in the string, using the lookup table values.
	while(len--)
		crc = (crc >> 8) ^ m_crc32_table[(crc & 0xFF) ^ *buffer++];
	// Exclusive OR the result with the beginning value.
	return (closure) ? crc^0xffffffff : crc;
}

// CRC 16bit berechnen
unsigned short CChannel3::GetCRC16(unsigned char *data, unsigned int size, unsigned short crc)
{
	unsigned int i;
	unsigned char j;
	//unsigned short crc;
  
	//crc = 0xFFFF;
    for (i=0; i < size; i++) {

        crc = crc ^ data[i];

        for (j=0; j < 8; j++) {
            if (crc & 0x1) {
				crc >>= 1; 
				crc = crc^0xA001;
			} else {
				crc >>= 1;
			}
        }
    }

    //data[size    ] = (unsigned char)(crc & 0xFF);
    //data[size + 1] = (unsigned char)(crc >> 8);
	return crc;
}

void CChannel3::MkPaketCrc(unsigned char *buf,int cnt)	
// CRC berechnen und an das Ende des Buffers anhngen
{
	int i,j;
	unsigned short int crc;
  
	crc = 0xFFFF;
    for (i=0;i<cnt;i++) {
        crc = crc ^ buf[i];
        for (j=0;j<8;j++) {
            if (crc & 0x1) {crc >>= 1; crc = crc^0xA001;} else crc >>= 1;
        }
    }
    buf[cnt] = (unsigned char)(crc&0xFF);
    buf[cnt+1] = (unsigned char)(crc>>8);
}

int CChannel3::CheckPaketCrc(unsigned char *buf, int cnt)
{
    unsigned short int crc;
    int i,j;

    crc = 0xFFFF;
    for (i=0;i<cnt-2;i++) {
        crc = crc ^ buf[i];
        for (j=0;j<8;j++) {
            if (crc & 0x1) {crc >>= 1; crc = crc^0xA001;} else crc >>= 1;
        }
    }
    if (buf[cnt-2]!=(unsigned char)(crc&0xFF)) return FALSE;
    if (buf[cnt-1]!=(unsigned char)(crc>>8)) return FALSE;
    return TRUE;
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// Firmwareversion lesen
int CChannel3::Ping(unsigned short deviceIDRemote)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	ResetError();

	m_result = Request(deviceIDRemote);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, PING);
	} else {

		// OK
	}

	return m_result;
}

// Geraetetyp ermitteln
int CChannel3::GetDeviceType(unsigned short deviceIDRemote, unsigned char &type)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_DEVICE_TYPE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_DEVICE_TYPE);
	} else {

		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (%i) Type.\n", m_data[0]);
			
			type = m_data[0];
		}
	}

	return m_result;
}

// Boardtyp ermitteln
int CChannel3::GetBoardType(unsigned short deviceIDRemote, unsigned char &type)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_BOARD_TYPE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_BOARD_TYPE);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (0x%04X) Boardtype.\n", m_data[0]);

			type = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

// Komponente ermitteln
int CChannel3::GetComponentRS485_0(unsigned short deviceIDRemote, unsigned char &value)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_RS485_0, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_RS485_0);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (0x%02X) Komponentenwert.\n", m_data[0]);

			value = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

// Komponente ermitteln
int CChannel3::GetComponentTCPIP(unsigned short deviceIDRemote, unsigned char &value)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_TCPIP, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_TCPIP);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (0x%02X) Komponentenwert.\n", m_data[0]);

			value = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

// Komponente ermitteln
int CChannel3::GetComponentModemMC35i(unsigned short deviceIDRemote, unsigned char &value)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_MODEM_MC35i, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_MODEM_MC35i);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (0x%02X) Komponentenwert.\n", m_data[0]);

			value = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

// Komponente ermitteln
int CChannel3::GetComponentRS485_1(unsigned short deviceIDRemote, unsigned char &value)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_RS485_1, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_HARDWARE, GET_COMPONENT_RS485_1);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangener (0x%02X) Komponentenwert.\n", m_data[0]);

			value = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

//#########################################################
//####################### SYSTEM ##########################
//#########################################################

// Systemreset ausfuehrenFirmwareversion lesen
int CChannel3::SystemReset(unsigned short deviceIDRemote, unsigned short modus)
{
	unsigned short count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Den Resetmodus uebernehmen.
	*(unsigned short*)m_data = modus;

	m_result = Execute(deviceIDRemote,MODULE_SYSTEM , SYSTEM_RESET, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SYSTEM_RESET);
	} else {


		// OK
		if (count != sizeof(unsigned short)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else if (*(unsigned short*)m_data != modus) {

			GetChannelBase().Trace("Gesendeter (%i) und Empfangener (%i) Modus ungleich.\n", modus, *(unsigned short*)m_data);
		}
	}

	return m_result;
}

// Systemmodus stellen
int CChannel3::BeginSegment(unsigned short deviceIDRemote, unsigned short art, unsigned int &address, unsigned int size, unsigned int crc)
{
	unsigned short count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;


	// Beginn einer Uebertragung eines Datenblocks

	// addresse wegen wohlgeformtem Paket mitgeben
	*(unsigned int*)&m_data[sizeof(unsigned short)] = address;
	count += sizeof(unsigned int);

	// geforderte Speichergroesze mitgeben
	*(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)] = size;
	count += sizeof(unsigned int);

	// CRC32 fuer Datenblock mitgeben
	*(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)*2] = crc;
	count += sizeof(unsigned int);

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, BEGIN_SEGMENT, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, BEGIN_SEGMENT);
	} else {


		// OK
		
		// Ausgeben der empfangenen Werte

		if (count == 14) {

			if (art != *(unsigned short*)m_data) GetChannelBase().Trace("ERROR: art\n");
			
			address = *(unsigned int*)&m_data[sizeof(unsigned short)];

			if (size != *(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)]) GetChannelBase().Trace("ERROR: size\n");
			//size    = *(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)];

			if (crc != *(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)*2]) GetChannelBase().Trace("ERROR: crc\n");
			//crc     = *(unsigned int*)&m_data[sizeof(unsigned short) + sizeof(unsigned int)*2];

			GetChannelBase().Trace("(0x%04X)(0x%08X)(0x%08X)(0x%08X)\n", art, address, size, crc);
		} else {

			GetChannelBase().Trace("Fehlerhafte Parameteranzahl.\n");
		}

		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Bei (den einzelnen innenliegenden) Systemstopps,
		// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
		if (art == SS_UPDATE) {

			unsigned short modul, function;
			// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
			// dieses musz hier seperat abgegriffen werden.
			if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

				GetChannelBase().Trace("Quittierungsprfantwort - CRC schlug fehl.\n");
				return m_result;
			}

			if ((modul == MODULE_SYSTEM) && (function == BEGIN_SEGMENT_OK)) {

				GetChannelBase().Trace("Antwort von CRC OK.\n");
			}

			if (*(unsigned short*)m_data != art) {

				GetChannelBase().Trace("Die Art der Quittierung ist ungleich.\n");

			} 
			
			if (count > sizeof(unsigned short)) {

				GetChannelBase().Trace("Datenmenge ist mit (%i) groszer als erwartet.\n", count);
			}

			GetChannelBase().Trace("PACKET_LAST: [");
			for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

				GetChannelBase().Trace("%02X", m_receive[i]);
				if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
				} else {
					GetChannelBase().Trace(" ");
				}
			}
			GetChannelBase().Trace("]\n");
		}

	}

	return m_result;
}

// Systemmodus stellen
int CChannel3::EndSegment(unsigned short deviceIDRemote, unsigned short art, unsigned int crc)
{
	unsigned short count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;

	// Beendigung einer Uebertragung eines Datanblocks
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, END_SEGMENT, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, END_SEGMENT);
	} else {


		// OK

		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Bei (den einzelnen innenliegenden) Systemstopps,
		// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
//		if (art != SS_UPDATE) {

			unsigned short modul, function;
			// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
			// dieses musz hier seperat abgegriffen werden.
			if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

				GetChannelBase().Trace("Quittierungsprfantwort - CRC schlug fehl (%i).\n", m_result);
				return m_result;
			}

			if ((modul == MODULE_SYSTEM) && (function == END_SEGMENT_OK)) {

				GetChannelBase().Trace("Antwort von CRC OK.\n");
			}

			if (*(unsigned short*)m_data != art) {

				GetChannelBase().Trace("Die Art der Quittierung ist ungleich.\n");

			} else if (*(unsigned int*)&m_data[sizeof(unsigned short)] != crc) {

				GetChannelBase().Trace("Der CRC der Quittierung ist ungleich. Soll (0x%08X), Ist (0x%08X)\n", crc, *(unsigned int*)&m_data[sizeof(unsigned short)]);
				return -IV_ERROR_CRC_PACKET;
			}

			GetChannelBase().Trace("PACKET_LAST: [");
			for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

				GetChannelBase().Trace("%02X", m_receive[i]);
				if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
				} else {
					GetChannelBase().Trace(" ");
				}
			}
			GetChannelBase().Trace("]\n");
//		}
	}

	return m_result;
}


#define PROTOCOL_SIZE_DATA 256

// Flash schreiben
int CChannel3::SetFlash(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned int count)
{
	unsigned int addressOffset;
	unsigned short i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += PROTOCOL_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + PROTOCOL_SIZE_DATA) > count) ? (unsigned short)(count - addressOffset) : PROTOCOL_SIZE_DATA;

		// Memberpuffer aufbereiten
		*(unsigned int*)m_data = address + addressOffset;
		// Zu setzende Daten uebernehmen
		for (i=0; i < sizeData; i++) {

			m_data[sizeof(unsigned int) + i] = buf[addressOffset + i];
		}

		// Variable aufbereiten
		sizeData += sizeof(unsigned int);

		m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_FLASH, m_data, sizeData);

		if (m_result != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, MODULE_SYSTEM, SET_FLASH);
			return m_result;
		} else {


			// OK
			//GetChannelBase().Trace("Address: 0x%08X, wrote: 0x%04X Byte\n", *(unsigned int*)m_data, *(unsigned short*)&m_data[sizeof(unsigned int)]);
			if (m_callBack1) {
				if (!m_callBack1()) {

					m_result = -IV_ERROR_BREAK;
					// Fehlernummern setzen
					m_errorCommunicationNum = IV_ERROR_BREAK;
					// Fehlertext ermitteln und eintragen.
					GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

					GetChannelBase().Trace("ERROR:( %04X )[%s]\n", -m_result & 0xFFFF, m_errorCommunication);
					break;
				}
			} else if (m_callBack2) {
				if (!m_callBack2(count, addressOffset + sizeData)) {

					m_result = -IV_ERROR_BREAK;
					// Fehlernummern setzen
					m_errorCommunicationNum = IV_ERROR_BREAK;
					// Fehlertext ermitteln und eintragen.
					GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

					GetChannelBase().Trace("ERROR:( %04X )[%s]\n", -m_result & 0xFFFF, m_errorCommunication);
					break;
				}
			}

			continue;
		}

		break;
	}

	return m_result;
}

// Flash lesen
int CChannel3::GetFlash(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned int count)
{
	unsigned int addressOffset;
	unsigned short i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += PROTOCOL_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + PROTOCOL_SIZE_DATA) > count) ? (unsigned short)(count - addressOffset) : PROTOCOL_SIZE_DATA;

		// Meberpuffer aufbereiten
		*(unsigned int*)m_data = address + addressOffset;
		// Angeforderte Laenge eintragen
		*(unsigned short*)&m_data[sizeof(unsigned int)] = sizeData;

		// Variable aufbereiten
		sizeData = sizeof(unsigned int) + sizeof(unsigned short);

		m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_FLASH, m_data, sizeData);

		if (m_result != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, MODULE_SYSTEM, GET_FLASH);
			return m_result;
		} else {


			// OK
			// Empfangene Daten uebernehmen
			for (i=0; i < (sizeData - sizeof(unsigned int)); i++) {

				buf[addressOffset + i] = m_data[sizeof(unsigned int) + i];
			}

			continue;
		}

		break;
	}

	return m_result;
}

// EEPROM schreiben
int CChannel3::SetEEPROM(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count)
{
	unsigned int addressOffset;
	unsigned short i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += PROTOCOL_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + PROTOCOL_SIZE_DATA) > count) ? (unsigned short)(count - addressOffset) : PROTOCOL_SIZE_DATA;

		// Memberpuffer aufbereiten
		*(unsigned int*)m_data = address + addressOffset;
		// Zu setzende Daten uebernehmen
		for (i=0; i < sizeData; i++) {

			m_data[sizeof(unsigned int) + i] = buf[addressOffset + i];
		}

		// Variable aufbereiten
		sizeData += sizeof(unsigned int);

		m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_EEPROM, m_data, sizeData);

		if (m_result != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, MODULE_SYSTEM, SET_EEPROM);
			return m_result;
		} else {

			// OK
			//GetChannelBase().Trace("Address: 0x%08X, wrote: 0x%04X Byte\n", *(unsigned int*)m_data, *(unsigned short*)&m_data[sizeof(unsigned int)]);
			if (m_callBack1) {
				if (!m_callBack1()) {

					// Fehlernummern setzen
					m_errorCommunicationNum = IV_ERROR_BREAK;
					// Fehlertext ermitteln und eintragen.
					GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

					GetChannelBase().Trace("ERROR:( %04X )[%s]\n", -m_result & 0xFFFF, m_errorCommunication);
				}
			} else if (m_callBack2) {
				if (!m_callBack2(count, addressOffset + sizeData)) {

					// Fehlernummern setzen
					m_errorCommunicationNum = IV_ERROR_BREAK;
					// Fehlertext ermitteln und eintragen.
					GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

					GetChannelBase().Trace("ERROR:( %04X )[%s]\n", -m_result & 0xFFFF, m_errorCommunication);
				}
			}

			continue;
		}

		break;
	}

	return m_result;
}

// EEPROM lesen
int CChannel3::GetEEPROM(unsigned short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count)
{
	unsigned int addressOffset;
	unsigned short i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += PROTOCOL_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + PROTOCOL_SIZE_DATA) > count) ? (unsigned short)(count - addressOffset) : PROTOCOL_SIZE_DATA;

		// Meberpuffer aufbereiten
		*(unsigned int*)m_data = address + addressOffset;
		// Angeforderte Laenge eintragen
		*(unsigned short*)&m_data[sizeof(unsigned int)] = sizeData;

		// Variable aufbereiten
		sizeData = sizeof(unsigned int) + sizeof(unsigned short);

		m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_EEPROM, m_data, sizeData);

		if (m_result != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, MODULE_SYSTEM, GET_EEPROM);
			return m_result;
		} else {


			// OK
			// Empfangene Daten uebernehmen
			for (i=0; i < (sizeData - sizeof(unsigned int)); i++) {

				buf[addressOffset + i] = m_data[sizeof(unsigned int) + i];
			}

			continue;
		}

		break;
	}

	return m_result;
}

// Firmwareversion lesen
int CChannel3::GetVersion(unsigned short deviceIDRemote, char *version)
{
	unsigned short i, count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_VERSION, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_VERSION);
	} else {


		// OK
		for (i=0; i < count; i++) {

			version[i] = m_data[i];
		}
	}

	return m_result;
}

// Bootloaderversion lesen
int CChannel3::GetVersionBootloader(unsigned short deviceIDRemote, char *version)
{
	unsigned short i, count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_VERSION_BOOTLOADER, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_VERSION_BOOTLOADER);
	} else {


		// OK
		for (i=0; i < count; i++) {

			version[i] = m_data[i];
		}
	}

	return m_result;
}

// Seriennummer lesen
int CChannel3::GetSerial(unsigned short deviceIDRemote, unsigned int &serial)
{
	unsigned short count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_SERIAL, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_SERIAL);
	} else {


		// OK
		serial = *(unsigned int*)m_data;
		if (count > sizeof(unsigned int)) GetChannelBase().Trace("Serial: %i\n", count);
	}

	return m_result;
}

// Datum/Uhrzeit setzen/lesen
int CChannel3::SetRTC(unsigned short deviceIDRemote, char *time)
{
	unsigned short i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	count = 20;
	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	for (i=0; i < (count - 1); i++) {

		m_data[i] = time[i];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_RTC, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_RTC);
	} else {


		// OK
	}

	return m_result;
}

int CChannel3::GetRTC(unsigned short deviceIDRemote, char *time)
{
	unsigned short i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_RTC, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_RTC);
	} else {


		// OK
		for (i=0; i < count; i++) {

			time[i] = m_data[i];
		}
	}

	return m_result;
}

// Systemvariable setzen/lesen
int CChannel3::SetSystemVariable(unsigned short deviceIDRemote, char *name, char *value)
{
	unsigned short i, j, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_SYSTEM_VARIABLE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_SYSTEM_VARIABLE);
	} else {


		// OK
		if (count != strlen(name) || strcmp(name, (char*)m_data)) {

			GetChannelBase().Trace("ERROR_INNER: ( Der Name einer Systemvariablen wurde nicht zurckgeliefert oder ist ungleich. )\n");
		}
	}

	return m_result;
}

int CChannel3::GetSystemVariable(unsigned short deviceIDRemote, char *name, char *value)
{
	unsigned short i, j, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_SYSTEM_VARIABLE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_SYSTEM_VARIABLE);
	} else {


		// OK
		if (count <= strlen(name) || strcmp(name, (char*)m_data)) {

			GetChannelBase().Trace("ERROR_INNER: ( Der Name einer Systemvariablen wurde nicht zurckgeliefert oder ist ungleich. )\n");
		} else {

			// Wert uebernehmen
			i = strlen(name) + 1;
			for (j=i; i < count; i++) {

				value[i - j] = m_data[i];
			}
		}
	}

	return m_result;
}

// Relais setzen/lesen
int CChannel3::SetRelay(unsigned short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time)
{
	unsigned short i, j, count = 0;
	char value[32];
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	// Busnummer
	sprintf(value, "%i", busNum);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	// Relaisnummer
	sprintf(value, "%i", relayNum);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	// Status
	sprintf(value, "%i", status);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	// Time
	sprintf(value, "%i", time);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_RELAY, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_RELAY);
	} else {


		// OK
	}

	return m_result;
}

int CChannel3::GetRelay(unsigned short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time)
{
	unsigned short i, j, count = 0;
	char value[32];
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	// Busnummer
	sprintf(value, "%i", busNum);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	// Relaisnummer
	sprintf(value, "%i", relayNum);
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_RELAY, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_RELAY);
	} else {


		// OK
		// Namen uebernehmen
		i = strlen((char*)m_data) + 1;
		// Busnummer
		i += strlen((char*)&m_data[i]) + 1;
		// Relaisnummer
		i += strlen((char*)&m_data[i]) + 1;
		// Status
		// Wert uebernehmen
		for (j=i; i < j+strlen((char*)&m_data[j])+1; i++) {

			value[i - j] = m_data[i];
		}
		status = atoi(value);
		// Time
		// Wert uebernehmen
		for (j=i; i < j+strlen((char*)&m_data[j])+1; i++) {

			value[i - j] = m_data[i];
		}
		time = atoi(value);
	}

	return m_result;
}

// Systembios aufrufen
int CChannel3::EnterBios(unsigned short deviceIDRemote)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, ENTER_BIOS, m_data, count, 1);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, ENTER_BIOS);
	} else {


		// OK
		
		// Ausgeben der empfangenen Werte

		if (count == 0) {

			GetChannelBase().Trace("OK...\n");
		} else {

			GetChannelBase().Trace("Fehlerhafte Parameteranzahl.\n");
		}

		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Nach dem Wechsel ins Bios wird eine Bestaetigung gesendet.
		unsigned short modul, function;
		if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("Quittierungsantwort - schlug fehl.\n");
			return m_result;
		}

		if ((modul == MODULE_SYSTEM) && (function == ENTERED_BIOS)) {

			GetChannelBase().Trace("Antwort OK.\n");
		}

		GetChannelBase().Trace("PACKET_LAST: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

	}

	return m_result;
}

// Bios verlassen
int CChannel3::ExitBios(unsigned short deviceIDRemote)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Beendigung einer Uebertragung eines Datanblocks
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, EXIT_BIOS, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, EXIT_BIOS);
	} else {


		// OK

		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Nach dem Verlassen des Bios kommt eine Bestaetigungsanfrage.
		unsigned short modul, function;
		if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("Quittierungsantwort - schlug fehl (%i).\n", m_result);
			return m_result;
		}

		if ((modul == MODULE_SYSTEM) && (function == EXITED_BIOS)) {

			GetChannelBase().Trace("Antwort OK.\n");
		}

		GetChannelBase().Trace("PACKET_LAST: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");
	}

	return m_result;
}

// Interface setzen (RS232, TCPIP...)
int CChannel3::SetInterface(unsigned short deviceIDRemote, unsigned char type)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Den Geraetetyp uebernehmen.
	m_data[0] = type;
	count += sizeof(unsigned char);

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_INTERFACE, m_data, count);
//	m_result = -IV_FUNCTION_UNAVAILABLE;

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_INTERFACE);
	} else {


		// OK
	}

	return m_result;
}

// Interface ermitteln (RS232, TCPIP...)
int CChannel3::GetInterface(unsigned short deviceIDRemote, unsigned char &type)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_INTERFACE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_INTERFACE);
	} else {


		// OK
		if (count != sizeof(unsigned char)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangenes (0x%02X) Interface.\n", m_data[0]);

			type = (m_data[0] != 0xFF) ? m_data[0] : 0;
		}
	}

	return m_result;
}

// Baudrate setzen
int CChannel3::SetBaud(unsigned short deviceIDRemote, unsigned int br)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Den Geraetetyp uebernehmen.
	*(unsigned int*)&m_data[0] = br;
	count += sizeof(unsigned int);

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_BAUD, m_data, count);
//	m_result = -IV_FUNCTION_UNAVAILABLE;

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_BAUD);
	} else {


		// OK
	}

	return m_result;
}

// Baudrate ermitteln
int CChannel3::GetBaud(unsigned short deviceIDRemote, unsigned int &br)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_BAUD, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_BAUD);
	} else {


		// OK
		if (count != sizeof(unsigned int)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangene (%u) Baudrate.\n", ((unsigned int*)m_data)[0]);

			br = *(unsigned int*)m_data;
			switch(br) { // Bereich pruefen
			case 9600:
			case 14400:
			case 19200:
			case 38400:
			case 57600:
			case 115200:
				break;
			default:
				br = 38400;
			}
		}
	}

	return m_result;
}

// DeviceID setzen
int CChannel3::SetDeviceIDBios(unsigned short deviceIDRemote, unsigned short id)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Den Geraetetyp uebernehmen.
	*(unsigned short*)&m_data[0] = id;
	count += sizeof(unsigned short);

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, SET_DEVICE_ID, m_data, count);
//	m_result = -IV_FUNCTION_UNAVAILABLE;

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, SET_DEVICE_ID);
	} else {


		// OK
	}

	return m_result;
}

// DeviceID ermitteln
int CChannel3::GetDeviceIDBios(unsigned short deviceIDRemote, unsigned short &id)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, GET_DEVICE_ID, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, GET_DEVICE_ID);
	} else {


		// OK
		if (count != sizeof(unsigned short)) {

			GetChannelBase().Trace("Erwarteter Parameter nicht vorhanden. (%i)\n", count);
		} else {

			GetChannelBase().Trace("Empfangene (%i) DeviceID.\n", ((unsigned short*)m_data)[0]);

			id = *(unsigned short*)m_data;
			if (!((id >= 1 && id <= 31) || id == 254)) {

				id = 254;
			}
		}
	}

	return m_result;
}

// Bootloadermodus aktivieren
int CChannel3::EnterBootloader(unsigned short deviceIDRemote)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, ENTER_BOOTLOADER, m_data, count, 1);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, ENTER_BOOTLOADER);
	} else {


		// OK
	}

	return m_result;
}

// Wechsel in Monitor Mode des XPort
int CChannel3::EnterXPortMonitor(unsigned short deviceIDRemote, char *data)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, ENTER_XPORT_MONITOR, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, ENTER_XPORT_MONITOR);
	} else {


		// OK
		
		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Bei (den einzelnen innenliegenden) Systemstopps,
		// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
//		if (art == SS_UPDATE) {

			unsigned short modul, function;
			// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
			// dieses musz hier seperat abgegriffen werden.
			if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

				GetChannelBase().Trace("Quittierungsprfantwort - CRC schlug fehl.\n");
				return m_result;
			}

			if ((modul == MODULE_SYSTEM) && (function == ENTER_XPORT_MONITOR_OK)) {

				GetChannelBase().Trace("Antwort von CRC OK.\n");
			}

			// Daten uebernehmen
			for (i=0; i < count; i++) {

				data[i] = m_data[i];
			}

			GetChannelBase().Trace("PACKET_LAST: [");
			for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

				GetChannelBase().Trace("%02X", m_receive[i]);
				if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
				} else {
					GetChannelBase().Trace(" ");
				}
			}
			GetChannelBase().Trace("]\n");
//		}

	}

	return m_result;
}

// Kommando an XPort Monitor senden
int CChannel3::CommandXPort(unsigned short deviceIDRemote, char *buf, unsigned short &count)
{
	unsigned short i;
	
	// Umgebung zuruecksetzen
	Reset();

	// Daten fuer senden uebernehmen
	for (i=0; i < count; i++) {

		m_data[i] = buf[i];
	}

	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, CMD_XPORT, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, CMD_XPORT);
	} else {


		// OK

		// Daten von XPort uebernehmen
		for (i=0; i < count; i++) {

			buf[i] = m_data[i];
		}
	}

	return m_result;
}

// Monitor Mode vom XPort beenden.
int CChannel3::ExitXPortMonitor(unsigned short deviceIDRemote)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Beendigung einer Uebertragung eines Datanblocks
	m_result = Execute(deviceIDRemote, MODULE_SYSTEM, EXIT_XPORT_MONITOR, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_SYSTEM, EXIT_XPORT_MONITOR);
	} else {

		// OK

		int i;
		// Zuvor gesendetes Paket
		GetChannelBase().Trace("PACKET_SEND: [");
		for (i=0; i < *(unsigned short*)&m_transmit[4]; i++) {

			GetChannelBase().Trace("%02X", m_transmit[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Zuvor empfangenes Paket
		GetChannelBase().Trace("PACKET_PREV: [");
		for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

			GetChannelBase().Trace("%02X", m_receive[i]);
			if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
			} else {
				GetChannelBase().Trace(" ");
			}
		}
		GetChannelBase().Trace("]\n");

		// Bei (den einzelnen innenliegenden) Systemstopps,
		// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
//		if (art != SS_UPDATE) {

			unsigned short modul, function;
			// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
			// dieses musz hier seperat abgegriffen werden.
			if ((m_result = Pending(deviceIDRemote, modul, function, m_data, count)) != IV_ERROR_SUCCESS) {

				GetChannelBase().Trace("Quittierungsprfantwort - CRC schlug fehl (%i).\n", m_result);
				return m_result;
			}

			if ((modul == MODULE_SYSTEM) && (function == EXIT_XPORT_MONITOR_OK)) {

				GetChannelBase().Trace("Antwort von CRC OK.\n");
			}

			GetChannelBase().Trace("PACKET_LAST: [");
			for (i=0; i < *(unsigned short*)&m_receive[4]; i++) {

				GetChannelBase().Trace("%02X", m_receive[i]);
				if ((i == 0) || (i == 2) || (i == 4) || (i == 12) || (i == 15) || (i == 17)) {
				} else {
					GetChannelBase().Trace(" ");
				}
			}
			GetChannelBase().Trace("]\n");
//		}
	}

	return m_result;
}

//#########################################################
//##################### APPLICATION #######################
//#########################################################

int CChannel3::ClearRecords(unsigned short deviceIDRemote)
{
	unsigned short count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, CLEAR_RECORDS, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, CLEAR_RECORDS);
	} else {


		// OK
	}

	return m_result;
}

int CChannel3::SetRecord(unsigned short deviceIDRemote, unsigned char &status)
{
	unsigned short count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, SET_RECORD, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, SET_RECORD);
	} else {


		// OK

		switch(m_data[0]) {
		case 0:
		case 2:
			status = 0;
			break;
		case MB_ANSW_ACK_ON_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
		case MB_ANSW_TIMEOUT_ACK_RECORD:   // Quittung kam nach Timeout (fuer Online-Datensatz) 
			status = 1;
			break;
		case MB_ANSW_OLD_ACK_ON_RECORD:    // wiederholt gesendete Quittung 
		case MB_ANSW_OLD_TIMEOUT_ACK_RECORD: // wiederholte Quittung nach Timeout 
			status = 2;
			break;
		case MB_ANSW_ACK_OFF_RECORD:       // Quittung erhalten (fuer Offline-Datensatz) 
			status = 3;
			break;
		case MB_ANSW_OLD_ACK_OFF_RECORD:   // wiederholt gesendete Quittung 
			status = 4;
			break;
		}

	}

	return m_result;
}

int CChannel3::GetRecord(unsigned short deviceIDRemote, unsigned char *buf, unsigned short &count, unsigned char &status)
{
	unsigned short i;
	
	// Umgebung zuruecksetzen
	Reset();

	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, GET_RECORD, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, GET_RECORD);
	} else {


		// OK

		switch(m_data[0]) {
		case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
			status = 0;
			break;
		case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
			status = 1;
			break;
		case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
			status = 2;
			break;
		case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
			status = 3;
			break;
		case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
			status = 4;
			break;
		}

		for (i=0; i < count; i++) {

			buf[i] = m_data[i];
		}
	}

	return m_result;
}

// Displaynachicht senden
int CChannel3::ShowMessage(unsigned short deviceIDRemote, unsigned char time, unsigned char action, unsigned char audio, char *text)
{
	unsigned short i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	m_data[0] = time;
	m_data[1] = action;
	m_data[2] = audio;
	count = 3;
	// Namen uebernehmen
	count += strlen(text) + 1;
	for (i=0; i < (count - 3); i++) {

		m_data[3 + i] = text[i];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, SHOW_MESSAGE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, SHOW_MESSAGE);
	} else {


		// OK
	}

	return m_result;
}

// Systemvariable setzen/lesen
int CChannel3::SetVariable(unsigned short deviceIDRemote, char *name, char *value)
{
	unsigned short i, j, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, SET_VARIABLE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, SET_VARIABLE);
	} else {


		// OK
		if ((unsigned)(count - 1) != strlen(name) || strcmp(name, (char*)m_data)) {

			GetChannelBase().Trace("ERROR_INNER: ( Der Name einer Systemvariablen wurde nicht zurckgeliefert oder ist ungleich. )\n");
		}
	}

	return m_result;
}

int CChannel3::GetVariable(unsigned short deviceIDRemote, char *name, char *value)
{
	unsigned short i, j, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	
	m_result = Execute(deviceIDRemote, MODULE_APPLICATION, GET_VARIABLE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_APPLICATION, GET_VARIABLE);
	} else {


		// OK
		if (count <= strlen(name) || strcmp(name, (char*)m_data)) {

			GetChannelBase().Trace("ERROR_INNER: ( Der Name einer Systemvariablen wurde nicht zurckgeliefert oder ist ungleich. )\n");
		} else {

			// Wert uebernehmen
			i = strlen(name) + 1;
			for (j=i; i < count; i++) {

				value[i - j] = m_data[i];
			}
		}
	}

	return m_result;
}


//#########################################################
//####################### TABLE ###########################
//#########################################################


int CChannel3::GetTable(unsigned short deviceIDRemote, char *name, STable2 &table)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Name als Parameter eintragen
	count = sprintf((char*)m_data, "%s", name) + 1;

	m_result = Execute(deviceIDRemote, MODULE_TABLE, GET_TABLE, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_TABLE, GET_TABLE);
	} else {


		// OK
		if (count == sizeof(STable2)) {

			table = *(STable2*)m_data;
		} else {

			GetChannelBase().Trace("Die geforderte Strukturgroesze wurde nicht gesendet.");
		}
	}

	return m_result;
}

int CChannel3::GetField(unsigned short deviceIDRemote, char *name, STable2Field &field)
{
	unsigned short count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Name als Parameter eintragen
	count = sprintf((char*)m_data, "%s", name) + 1;

	m_result = Execute(deviceIDRemote, MODULE_TABLE, GET_FIELD, m_data, count);

	if (m_result != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, MODULE_TABLE, GET_FIELD);
	} else {


		// OK
		if (count == sizeof(STable2Field)) {

			field = *(STable2Field*)m_data;
		} else {

			GetChannelBase().Trace("Die geforderte Strukturgroesze wurde nicht gesendet.");
		}
	}

	return m_result;
}



//#########################################################
//####################### HELPER ##########################
//#########################################################

int CChannel3::Request(unsigned short deviceIDRemote)
{
	unsigned short transmitCount, receiveCount;
	int err;

	// Sendebuffer aufbereiten
	// Schicht 2
	//	Adressen
	*(unsigned short*)&m_transmit[0] = GetDeviceIDRemote();            // Empfaenger
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S2] = GetDeviceID(); // Absender
	
	// Paketlaenge
	transmitCount = V_KOPF+1+1+1+2+1+4+0+2; // Sendepaket
	receiveCount = 0;	                   // Empfangspaket

	*(unsigned short*)&m_transmit[V_O_LAENGE] = transmitCount;
	m_transmit[V_O_B_S2]         = B_S2_NO;
	m_transmit[V_O_PAKET_NR]     = 0;
	m_transmit[V_O_TIME_TO_LIVE] = 8;

	// Schicht 3
	m_transmit[V_O_B_S3]         = B_S3_NO;
	m_transmit[V_O_ANZ_BLOECKE]  = 1;
	m_transmit[V_O_AKT_BLOCK]    = 1;

	// Schicht 4
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S4] = GetDeviceID(); // Absender
	m_transmit[V_O_B_S4] = B_S4_NEWMB | BB_S4_ANFRAGE;      // Versionsbefehl
	*(unsigned short*)&m_transmit[V_O_ADD] = MODULE_HARDWARE;       // Modulnummer
	*(unsigned short*)&m_transmit[V_O_ADD + sizeof(unsigned short)] = PING; // Funktionsnummer

	MkPaketCrc(m_transmit, transmitCount-2);
	
	// Paket senden
	if ((err = Transmit(m_transmit, transmitCount)) != IV_ERROR_SUCCESS) {

		return err;
	}
	// Paket abholen
	if ((err = Receive(m_receive, receiveCount, 1)) != IV_ERROR_SUCCESS) {

		return err;
	}
	//	CRC pruefen
	if (!CheckPaketCrc(m_receive, receiveCount)) { 

		return -IV_ERROR_CRC_PACKET;
	}
	//	Empfaenger-/Absender-Adresse pruefen
	if (*(unsigned short*)&m_receive[0] != GetDeviceID() || 
		  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S2] != deviceIDRemote ||
		  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S4] != deviceIDRemote) {

		return -IV_ERROR_ADDRESS;
	}

	return IV_ERROR_SUCCESS;
}

int CChannel3::Execute(unsigned short deviceIDRemote, unsigned short modul, unsigned short function, unsigned char *buf, unsigned short &count, unsigned char retries)
// Modulbefehl senden und Antwort zurueckgeben
//  deviceIDRemote -> Adresse (Geraetenummer) des Empfaengers
//  modul    -> Nummer des anzusprechenden Modules
//  function -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> Modul-/Funktionsfehler
{
	unsigned char Versuch;
	unsigned short transmitCount, receiveCount, i;
	int err = IV_ERROR_SUCCESS;

	// Letzte Modul/Funktions Paar halten. (Fuer Fehlerauswertung)
	m_lastModule   = modul;
	m_lastFunction = function;

	// Sendebuffer aufbereiten
	// Schicht 2
	//	Adressen
	*(unsigned short*)&m_transmit[0] = deviceIDRemote;              // Empfaenger
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S2] = GetDeviceID(); // Absender
	
	// Paketlaenge
	transmitCount = V_KOPF+1+1+1+2+1+4+count+2;	// Sendepaket
	receiveCount = 0;	                        // Empfangspaket

	*(unsigned short*)&m_transmit[V_O_LAENGE] = transmitCount;
	m_transmit[V_O_B_S2]         = B_S2_NO;
	m_transmit[V_O_PAKET_NR]     = 0;
	m_transmit[V_O_TIME_TO_LIVE] = 8;

	// Schicht 3
	m_transmit[V_O_B_S3]         = B_S3_NO;
	m_transmit[V_O_ANZ_BLOECKE]  = 1;
	m_transmit[V_O_AKT_BLOCK]    = 1;

	// Schicht 4
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S4] = GetDeviceID(); // Absender
	m_transmit[V_O_B_S4] = B_S4_NEWMB | BB_S4_ANFRAGE;      // Versionsbefehl
	*(unsigned short*)&m_transmit[V_O_ADD] = modul;                 // Modulnummer
	*(unsigned short*)&m_transmit[V_O_ADD + sizeof(unsigned short)] = function; // Funktionsnummer

	for (i=0; i < count; i++) {

		m_transmit[i + V_O_DATEN] = buf[i];	// Daten in Sendebuffer kopieren
	}
	MkPaketCrc(m_transmit, transmitCount-2);
	
	// Daten lesen incl. Wiederholungen
	for (Versuch=0;Versuch < retries; Versuch++) {
		// Im Fehlerfall zur Sicherheit groe Anzahl Zeichen lesen um Empfangsbuffer
		// des Betriebssystem zu leeren
		if (Versuch >= 2) { GetChannelBase().Flush(); }
		// Paket senden 
		if ((err = Transmit(m_transmit, transmitCount)) != IV_ERROR_SUCCESS) {
			continue;
		}
		// Antwort abholen
		if ((err = Receive(m_receive, receiveCount)) != IV_ERROR_SUCCESS) {
			continue;
		}
		// Antwort auswerten
		//	CRC prfen
		if (!CheckPaketCrc(m_receive, receiveCount)) { 

			err = -IV_ERROR_CRC_PACKET;
			continue;
		}
		//	Empfaenger-/Absender-Adresse pruefen
		if (*(unsigned short*)&m_receive[0] != GetDeviceID() || 
			  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S2] != deviceIDRemote ||
			  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S4] != deviceIDRemote) {

			err = -IV_ERROR_ADDRESS;
			continue;
		}
		//	Laengenfeld pruefen
		if (*(unsigned short*)&m_receive[V_O_LAENGE] != receiveCount) {

			err = -IV_ERROR_PACKETLENGTH;
			continue;
		}

		// Anzahl der gelesenen Datenbyte berechnen
		count = receiveCount - (V_KOPF+1+1+1+2+1+4+2);

		//	Befehl Schicht 4 pruefen
		switch(m_receive[V_O_B_S4]) {
		case (B_S4_NEWMB|BB_S4_ANTWORT):

			// Daten in Buffer der Applikation kopieren
			for(i=0; i< count; i++) buf[i] = m_receive[i + V_O_DATEN];
			err = IV_ERROR_SUCCESS;
			break;

		case (B_S4_NEWMB|BB_S4_ERROR):
			// Fehlercode uebergeben
			err = *(unsigned short*)&m_receive[V_O_DATEN];
			break;

		default:
			err = -IV_ERROR_COMMAND;
			continue;
		}

		// Schleife nach erfolgreicher Abarbeitung verlassen.
		break;
	}

	return err;
}

int CChannel3::Pending(unsigned short deviceIDRemote, unsigned short &modul, unsigned short &function, unsigned char *buf, unsigned short &count, unsigned char retries)
// Modulbefehl Empfangen und Antwort zurueckgeben
//  deviceIDRemote -> Adresse (Geraetenummer) des Empfaengers
//  modul    -> Nummer des anzusprechenden Modules
//  function -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> Modul-/Funktionsfehler
{
	unsigned short Versuch, transmitCount, receiveCount, i;
	int err = IV_ERROR_SUCCESS;

	// Daten lesen incl. Wiederholungen
	for (Versuch=0; Versuch < retries; Versuch++) {

		// Anfrage abholen
		if ((err = Receive(m_receive, receiveCount)) != IV_ERROR_SUCCESS) {
			continue;
		}
		// Anfrage auswerten
		//	CRC pruefen
		if (!CheckPaketCrc(m_receive, receiveCount)) { 

			err = -IV_ERROR_CRC_PACKET;
			continue;
		}
		//	Empfaenger-/Absender-Adresse pruefen
		if (*(unsigned short*)&m_receive[0] != GetDeviceID() || 
			  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S2] != deviceIDRemote ||
			  *(unsigned short*)&m_receive[V_O_AD_ABSENDER_S4] != deviceIDRemote) {

			err = -IV_ERROR_ADDRESS;
			continue;
		}
		//	Laengenfeld pruefen
		if (*(unsigned short*)&m_receive[V_O_LAENGE] != receiveCount) {

			err = -IV_ERROR_PACKETLENGTH;
			continue;
		}

		// Anzahl der Datenbyte berechnen
		count = receiveCount - (V_KOPF+1+1+1+2+1+4+2);

		//	Befehl Schicht 4 pruefen
		switch(m_receive[V_O_B_S4]) {
		case (B_S4_NEWMB|BB_S4_ANFRAGE):

			modul = *(unsigned short*)&m_receive[V_O_ADD];
			function = *(unsigned short*)&m_receive[V_O_ADD + sizeof(unsigned short)];

			m_transmit[V_O_B_S4] = B_S4_NEWMB | BB_S4_ANTWORT;      // Versionsbefehl

			// Daten in Buffer der Applikation kopieren
			for(i=0; i< count; i++) buf[i] = m_receive[i + V_O_DATEN];
			err = IV_ERROR_SUCCESS;
			break;

		default:
			m_transmit[V_O_B_S4] = B_S4_NEWMB | BB_S4_ERROR;      // Versionsbefehl
			err = IV_ERROR_SUCCESS;
			//continue;
		}

		// Schleife nach erfolgreicher Abarbeitung verlassen.
		break;
	}
	if (Versuch == A_VERSUCHE_PENDING) {

		GetChannelBase().Trace("Zu viele Leseversuche.\n");
		return -IV_ERROR_UNREACHABLE;
	}

	// Sendebuffer aufbereiten
	// Schicht 2
	//	Adressen
	*(unsigned short*)&m_transmit[0] = deviceIDRemote;              // Empfaenger
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S2] = GetDeviceID(); // Absender
	
	// Paketlaenge
	transmitCount = V_KOPF+1+1+1+2+1+4+ 0 +2;	// Sendepaket
	receiveCount = 0;	                        // Empfangspaket

	*(unsigned short*)&m_transmit[V_O_LAENGE] = transmitCount;
	m_transmit[V_O_B_S2]         = B_S2_NO;
	m_transmit[V_O_PAKET_NR]     = 0;
	m_transmit[V_O_TIME_TO_LIVE] = 8;

	// Schicht 3
	m_transmit[V_O_B_S3]         = B_S3_NO;
	m_transmit[V_O_ANZ_BLOECKE]  = 1;
	m_transmit[V_O_AKT_BLOCK]    = 1;

	// Schicht 4
	*(unsigned short*)&m_transmit[V_O_AD_ABSENDER_S4] = GetDeviceID(); // Absender
	//m_transmit[V_O_B_S4] = B_S4_NEWMB | BB_S4_ANFRAGE;      // Versionsbefehl
	*(unsigned short*)&m_transmit[V_O_ADD] = modul;                 // Modulnummer
	*(unsigned short*)&m_transmit[V_O_ADD + sizeof(unsigned short)] = function; // Funktionsnummer
/*
	for (i=0; i < count; i++) {

		m_transmit[i + V_O_DATEN] = buf[i];	// Daten in Empfangsbuffer kopieren
	}
*/
	MkPaketCrc(m_transmit, transmitCount-2);
	
	if ((err = Transmit(m_transmit, transmitCount)) != IV_ERROR_SUCCESS) {
		
		return err;
	}

	return err;
}

// Paket versenden
int CChannel3::Transmit(unsigned char *buf, unsigned short count, int transmitTrials)
{
	ULONG nb;

	if (count) {
		if (GetChannelBase().TransmitData(buf, count, (int&)nb, transmitTrials) != IV_ERROR_SUCCESS) { return -IV_ERROR_TRANSMIT;}
		if (nb != count) { return -IV_ERROR_TRANSMIT;}
	}

	return IV_ERROR_SUCCESS;
}

// Paket empfangen
int CChannel3::Receive(unsigned char *buf, unsigned short &count, int receiveTrials)
{
	ULONG nb;

	// Protokoll-Kopf lesen
	count = V_KOPF;
	if (GetChannelBase().ReceiveData(buf, count, (int&)nb, receiveTrials) != IV_ERROR_SUCCESS) {

		GetChannelBase().Trace("ERROR_00: Read(0, %i, %i)\n", count, nb);
		return -IV_ERROR_RECEIVE;
	}
	if (nb == 0) {

		GetChannelBase().Trace("ERROR_01: Read(0, %i, %i)\n", count, nb);
		return -IV_ERROR_UNREACHABLE;
	}
	if (nb < V_KOPF) {

		// Noch nicht alle Byte gelesen -> versuchen weitere Bytes zu lesen
		ULONG lnb;
		if (GetChannelBase().ReceiveData(&buf[nb], count - nb, (int&)lnb, receiveTrials) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("ERROR_02: Read(0, %i, %i)\n", count - nb, lnb);
			return -IV_ERROR_RECEIVE;
		}
		nb += lnb;
	}
	if (nb != (unsigned int)count) {

		GetChannelBase().Trace("ERROR_03: Read(0, %i, %i)\n", count, nb);
		return -IV_ERROR_RECEIVE;
	}

	//  Laenge des Paketes aus Paketkopf ermitteln
	count = *(unsigned short*)&buf[V_O_LAENGE];

	// Laenge auf plausibilitaet pruefen
	if ((count < 21) || (count > V_MAX_PAKET)) {

		GetChannelBase().Trace("ERROR_04: %i\n", count);
		return -IV_ERROR_PACKETLENGTH;
	}

	// Die restlichen Bytes lesen
	if (GetChannelBase().ReceiveData(&buf[V_KOPF], count - V_KOPF, (int&)nb, receiveTrials) != IV_ERROR_SUCCESS) {

		GetChannelBase().Trace("ERROR_05: Read(0, %i, %i)\n", count - V_KOPF, nb);
		return -IV_ERROR_RECEIVE;
	}
	if (nb == 0) { 

		GetChannelBase().Trace("ERROR_06: Read(0, %i, %i)\n", count, nb);
		return -IV_ERROR_UNREACHABLE; 
	}
	if (nb < (ULONG)(count - V_KOPF)) {

		// Noch nicht alle Byte gelesen -> versuchen weitere Bytes zu lesen
		ULONG lnb;
		if (GetChannelBase().ReceiveData(&buf[nb], count - nb, (int&)lnb, receiveTrials) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("ERROR_07: Read(0, %i, %i)\n", count - nb, lnb);
			return -IV_ERROR_RECEIVE;
		}
		nb += lnb;
	}
	if (nb != (unsigned short)(count - V_KOPF)) {
 
		GetChannelBase().Trace("ERROR_08: %i != %i\n", nb, count - V_KOPF);
		return -IV_ERROR_RECEIVE;
	}

	return IV_ERROR_SUCCESS;
}

// Fehlerzustaende zuruecksetzen
void CChannel3::ResetError()
{
	// Fehlernummern loeschen
	m_errorCommunicationNum = m_errorModuleNum = m_errorFunctionNum = 0;
	// Puffer die in Gebrauch waren, loeschen.
	if (m_errorCommunication[0]) memset(m_errorCommunication, 0, sizeof(m_errorCommunication));
	if (m_errorModule[0]) memset(m_errorModule, 0, sizeof(m_errorModule));
	if (m_errorFunction[0]) memset(m_errorFunction, 0, sizeof(m_errorFunction));
}

void CChannel3::TraceError(unsigned short deviceIDRemote, unsigned short module, unsigned short function)
{
	if (m_result < 0) {
		
		// Fehlernummern setzen
		m_errorCommunicationNum = -m_result;
		// Fehlertext ermitteln und eintragen.
		GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

		GetChannelBase().Trace("ERROR:( %04X )[%s]\n", -m_result & 0xFFFF, m_errorCommunication);
	} else if (m_result > 0) {


		// Fehlertext ermitteln und eintragen.
		if ((m_result >> 8) & 0xFF) {

			// Fehlernummern setzen
			m_errorModuleNum = (m_result >> 8) & 0xFF;
			// Modulefehlertext ermitteln
			GetErrorModule(module, m_errorModuleNum, m_errorModule);

			GetChannelBase().Trace("ERROR:( %04X )[%s]\n", m_result & 0xFFFF, m_errorModule);
		} else {

			// Fehlernummern setzen
			m_errorFunctionNum = m_result & 0xFF;
			// Funktionsfehlertext ermitteln
			GetErrorFunction(module, function, m_errorFunctionNum, m_errorFunction);

			GetChannelBase().Trace("ERROR:( %04X )[%s]\n", m_result & 0xFFFF, m_errorFunction);
		}
	}
}

// Einen Kommunikationsfehler in seine Textbedeutung wandeln.
void CChannel3::GetErrorCommunication(unsigned short error, char *text)
{
	switch(error) {
	case IV_ERROR_SUCCESS: sprintf(text, "( %i ): Kein Fehler", error); break;
	case IV_ERROR_CHANNEL_OPEN: sprintf(text, "( %i ): Schnittstelle konnte nicht geffnet werden.", error); break;
	case IV_ERROR_UNREACHABLE: sprintf(text, "( %i ): Gert antwortet nicht auf Anfrage.", error); break;
	case IV_ERROR_TRANSMIT: sprintf(text, "( %i ): Es konnten keine Daten in die Schnittstelle geschrieben werden.", error); break;
	//case IV_ERROR_RECEIVE: sprintf(text, "( %i ): Es konnten keine Daten von der Schnittstelle gelesen werden.", error); break;
	case IV_ERROR_FILE_OPEN: sprintf(text, "( %i ): Datei konnte nicht geffnet werden.", error); break;
	case IV_ERROR_CRC_PACKET: sprintf(text, "( %i ): Prfsumme (CRC) von Protokollpaket stimmt nicht.", error); break;
	case IV_ERROR_ADDRESS: sprintf(text, "( %i ): Die Adressen (Sender/Empfnger) des Protokollpaketes sind fehlerhaft.", error); break;
	case IV_ERROR_COMMAND: sprintf(text, "( %i ): Protokollbefehl der vierten Schicht ist falsch oder unbekannt.", error); break;
	//case IV_ERROR_PACKETLENGTH: sprintf(text, "( %i ): Das geforderte Datenformat wurde nicht eingehalten.", error); break; // Datensatz-Format falsch im  TB
	case IV_ERROR_BREAK: sprintf(text, "( %i ): Die Kommunikation wurde durch den Anwender (CallBack-Funktion) abgebrochen.", error); break; // Daten lesen von Anwender abgebrochen weil Datei bereits vorhenden
	case IV_ERROR_FUNCTION_UNAVAILABLE: sprintf(text, "( %i ): Unbekannt", error); break;
	default: sprintf(text, "( %i ): Nicht eingetragener Fehler!", error); break;
	}
}

// Einen Modulefehler in seine Textbedeutung wandeln.
void CChannel3::GetErrorModule(unsigned short module, unsigned char error, char *text)
{
	// Die ersten vier Fehler sind in allen Modulen die selben.
	switch(error) {
	case 1: sprintf(text, "( %i ): Das geforderte Modul wird nicht untersttzt.", error); return;
	case 2: sprintf(text, "( %i ): Die geforderte Funktion wird nicht untersttzt.", error); return;
	case 3: sprintf(text, "( %i ): Es wurden bei der Funktion zu wenige Paramteter angegeben.", error); return;
	case 4: sprintf(text, "( %i ): Es wurden bei der Funktion zu viele Paramteter angegeben.", error); return;
	}

	switch(module) {
	case MODULE_SYSTEM:
		switch(error) {
		case 5: //-> Zu wenig Daten
		case 6: //-> Zu viele Daten
		default: sprintf(text, "( %i ): Nicht eingetragener Fehler!", error); break;
		}
		break;
	case MODULE_APPLICATION:
		switch(error) {
		case 0:
		default: sprintf(text, "( %i ): Nicht eingetragener Fehler!", error); break;
		}
		break;
	case MODULE_TABLE:
		switch(error) {
		case 0:
		default: sprintf(text, "( %i ): Nicht eingetragener Fehler!", error); break;
		}
		break;
	default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Module!", error); break;
	}
}

// Einen Funktionsfehler in seine Textbedeutung wandeln.
void CChannel3::GetErrorFunction(unsigned short module, unsigned short function, unsigned char error, char *text)
{
	switch(module) {
	case MODULE_HARDWARE:
		switch(function) {
		case PING:
			sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error);
			break;
		}
		break;
	case MODULE_SYSTEM:
		switch(function) {
		case SYSTEM_RESET:
			switch(error) {
			case 1: sprintf(text, "( %i ): Resetmodus unbekannt.", error); break;
			}
			break;
		case SET_SERIAL:
			switch(error) {
			case 1: sprintf(text, "( %i ): Pruefung der Seriennummer - Verschluesselung schlug fehl.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case GET_VERSION:
		case GET_VERSION_BOOTLOADER:
		case GET_SERIAL:
		case SET_RTC:
		case GET_RTC:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case SET_SYSTEM_VARIABLE:
		case GET_SYSTEM_VARIABLE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der zu setzende Variablenwert ist zu lang.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case SET_RELAY:
		case GET_RELAY:
			switch(error) {
			case 1:sprintf(text, "( %i ): Bereichsname unbekannt. (INTERN, TMR, IOM ...)", error); break;
			case 2:sprintf(text, "( %i ): Busnummer (z.B. TMR) nicht bekannt oder erreichbar.", error); break;
			case 3:sprintf(text, "( %i ): Relaisnummer unbekannt.", error); break;
			case 4:sprintf(text, "( %i ): Zu setzender Zustand (Status) unbekannt oder nicht untersttzt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case SET_FLASH:
		case GET_FLASH:
		case SET_EEPROM:
		case GET_EEPROM:
		case SET_PROGRAM:
		case GET_PROGRAM:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case BEGIN_SEGMENT:
		case END_SEGMENT:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Typ ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Die Updatesegmente sind erschpft.", error); break;
			case 3:sprintf(text, "( %i ): Der Angeforderte Speicherplatz steht nicht zur Verfgung.", error); break;
			case 4:sprintf(text, "( %i ): Es wurde kein STOP vorm START durchgefhrt.", error); break;
			case 5:sprintf(text, "( %i ): Verschachtelungsfehler von STOP und START.", error); break;
			case 6:sprintf(text, "( %i ): Prfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener berein.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case BEGIN_SEGMENT_OK:
		case END_SEGMENT_OK:
			break;
		case ENTER_BIOS:
		case EXIT_BIOS:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case ENTERED_BIOS:
		case EXITED_BIOS:
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		break;
	case MODULE_APPLICATION:
		switch(function) {
		case CLEAR_RECORDS:
		case GET_RECORD:
		case SET_RECORD:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case SHOW_MESSAGE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Nachrichtentext zu lang fuer die Anzeige.", error); break;
			case 2:sprintf(text, "( %i ): Es wird bereits eine Nachricht angezeigt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case SET_VARIABLE:
		case GET_VARIABLE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der zu setzende Variablenwert ist zu lang.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		break;
	case MODULE_TABLE:
		switch(function) {
		case GET_TABLE:
		case GET_FIELD:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der Subminorbestandteil des Tabellenfeldnamens ist unbekannt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		break;
	default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Module!", error); break;
	}
}

void CChannel3::SetCBFunction(int (*pFunction)(void))
{
	m_callBack1 = pFunction;
}

void CChannel3::SetCBFunctionWithParams(int (*pFunction)(unsigned int, unsigned int))
{
	m_callBack2 = pFunction;
}
