#ifndef _CHANNEL4_H_
#define _CHANNEL4_H_

// Basisklasse einbinden
#include "Channel4ModuleTable.h"

// Defines setzen, wenn nicht bereits geschehen
#ifndef _WIN32

#ifndef WINAPI
#define WINAPI __stdcall
#endif

typedef void *LPVOID;
typedef unsigned int DWORD;

#endif

/*
#define ICMP_ECHO       8 
#define ICMP_ECHOREPLY  0 

#define ICMP_MIN        8 // minimum 8 byte icmp packet (just header) 

#define DEF_PACKET_SIZE 32 

// The IP header 
typedef struct iphdr {
	unsigned int h_len:4;          // length of the header
	unsigned int version:4;        // Version of IP
	unsigned char tos;             // Type of service
	unsigned short total_len;      // total length of the packet
	unsigned short ident;          // unique identifier
	unsigned short frag_and_flags; // flags
	unsigned char  ttl;
	unsigned char proto;           // protocol (TCP, UDP etc)
	unsigned short checksum;       // IP checksum

	unsigned int sourceIP;
	unsigned int destIP;
} IpHeader;

//
// ICMP header
//
typedef struct _ihdr {
	BYTE i_type;
	BYTE i_code; // type sub code 
	USHORT i_cksum;
	USHORT i_id;
	USHORT i_seq;
	// This is not the std header, but we reserve space for time 
	ULONG timestamp;
} IcmpHeader;
*/

class tcpipInfo 
{
public:
	tcpipInfo() 
	{
		Reset();
	}

	void Reset()
	{
		length = sizeof(data);
		memset(data, 0, sizeof(data));
		memset(ip, 0, sizeof(ip));
		version = mac = NULL;
		port = 0;
	}

	void Parse()
	{
		// Werte aufschluesseln
		int num = 0;
		unsigned char b1, b2, b3, b4;
		char *token = strtok(data, "\r"), *pIp = NULL;
		while (token != NULL) {
			
			num++;
			switch(num) {
			case 1: version = token; break;
			case 2:	pIp = token; break;
			case 3:	port = atoi(token); break;
			case 4:	mac = token; break;
			}
			
			token = strtok(NULL, "\r");
		}

		// Die IP- und Portnummern werden mit Vornullen geliefert,
		// Diese sind zu entfernen.
		num = 0;
		b1 = b2 = b3 = b4 = 0;
		if (pIp != NULL && strlen(pIp)) {

			token = strtok(pIp, ".");
			while (token != NULL) {

				num++;
				switch(num) {
				case 1: b1 = (unsigned char)atoi(token); break;
				case 2: b2 = (unsigned char)atoi(token); break;
				case 3: b3 = (unsigned char)atoi(token); break;
				case 4: b4 = (unsigned char)atoi(token); break;
				}

				token = strtok(NULL, ".");
			}
		}

		// Ip ohne Vornullen
		sprintf(ip, "%i.%i.%i.%i", b1, b2, b3, b4);

		// IP Pruefen
		if (inet_addr(ip) == INADDR_NONE) {

			// Ungueltige Adresse
			memset(ip, 0, sizeof(ip));
		}
	}
public:
	char data[255];
	int length;
	char *version;
	char ip[16];
	int port;
	char *mac;
};


class CChannel4 : public CChannel4ModuleTable
{
public:
	CChannel4() : CChannel4ModuleTable(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID)
	{ 
		InitClass(); 
	}
	CChannel4(short channelID, short deviceID) : CChannel4ModuleTable(channelID, deviceID) 
	{ 
		InitClass(channelID, deviceID); 
	}
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	virtual ~CChannel4();

	friend class CAEChannel;

	// Ist das Kanalobjekt im Wartungsmodus.
	inline void SetServiceState(int state) 
	{ 
		m_serviceState = state; 
	}
	inline int GetServiceState() 
	{ 
		return m_serviceState; 
	}

	// Verbindungsannahme von uebergebenem Listen-Socket
	int Connect(SOCKET *listenSocket, short channelID, short deviceID = IV_DEFAULT_DEVICEID, int timeout = 3000, int aliveTO = 10, int infoFlags = 0);
	// Endesignal im Workerthread setzen und auf dessen Beendigung warten
	void DirectClose();

// Main - Funktion fuer Befehlsausfuehrung in Workerthread
public:
	tcpipInfo m_tcpipInfo;
	int m_notify;
protected:
#ifdef _WIN32
	static unsigned long WINAPI CommunicationThread(LPVOID lpParam);
#else
	static void *CommunicationThread(void *lpParam);
#endif
	volatile int m_endSignal;
	volatile int m_initSignal;
#ifdef _WIN32
	HANDLE m_communicationThread;
	DWORD m_threadID;
#else
	pthread_t m_communicationThread;
#endif
	// Socket fuer Connect
	SOCKET *m_listenSocket;
	int m_aliveTO;
	int m_infoFlags;
	volatile enum threadCommand {
		idle,
		suspending,
		suspend,
		resuming,
		resume
	} m_threadCommand;
	volatile int m_suspendCounter;
	unsigned char m_serviceState;

public:
	// Ausfuehren eines Protokollbefehls
	virtual int Execute(short context, short module, short func, unsigned char *buf, int &count, int contextValue = -1);

public:
	// Beginn und Ende der Kommunikation ueber einen TimeboySlot
	int SetTimeboySlotState(short deviceID, int state, int commandResult = IV_ERROR_SUCCESS)
	{
		static clock_t stamp = 0;
		int res;
		if ((res = CChannel4Protocol::SetTimeboySlotState(deviceID, state, commandResult)) == IV_ERROR_SUCCESS)
		{
			// TimeboyIV zur Kommunikationsinitialisiserung einen Ping senden
			if (state && ((clock() - stamp) / CLOCKS_PER_SEC > 3) || GetPacketNumber() == 0)
			{
				if ((res = Ping(deviceID, NULL, 0)) == IV_ERROR_SUCCESS)
				{
					stamp = clock();
				}
			}
		}

		// Wenn es kein TimeboySlot ist, dann trotzdem OK melden.
		if (res == -IV_ERROR_SLOT_NONE)
		{
			res = IV_ERROR_SUCCESS;
		}

		return res;
	}

protected:
	//#################################
	// CallRoutinen fuer die Abarbeitung anstehender Kommunikationsbefehle
//	virtual int CallFunction(short module, short func, unsigned char *data, int &count);
	// NotifyRoutine fuer die Abarbeitung von Benachrichtigungen unter den Klassen
//	virtual int CallFunctionNotify(short module, short func, unsigned char *data, int &count);

/*
protected:
	char icmp_data[IV_PACKET_LENGTH_MAX];
	char recvbuf[IV_PACKET_LENGTH_MAX];

public:
	int PingICMP(char *remote);
	int BroadcastXPort(char *address_net);
	int CmdXPort(char *remote, char *data, unsigned short &count, int needAnswer = TRUE);
	void PreparePacket(char * icmp_data, int datasize);
	void CheckPacket(char *buf, int bytes, struct sockaddr_in *from);
	unsigned short Checksum(unsigned short *buffer, int size);
*/
};

#endif
