/** 
 * @file   aes.h
 * @author Bernd Ottmann
 * @date   15.02.2002
 * @brief  Auflistung der Standard- definitionen und Strukturen
 */
#ifndef _SETUP_H_
#define _SETUP_H_

#ifdef _WIN32
#pragma warning(disable : 4996)
#endif

#pragma pack(1)

/////////////////////////////////////////////////////////////////////
// Angaben ueber verwendete Speichergroessen / Startadressen

#define EEP_SIZE             0x1000   ///< Gesamtgroesse vom EEPROM des Mikrokontroller (4kByte) 
#define EEP_SIZE_SU          0x0200   ///< Groesse des EEPROM die in das Geraet uebertragen wird (512Byte) 
#define EEP_SIZE_PT          0x2C     ///< Reservierter EEPROM - Bereich fuer Pointereintraege (ersten 44Byte)
#define FLASH_SIZE_SU        0x4000   ///< Groesse des FLASH - Bereichs fuer Setupdaten (16kByte) 
#define FLASH_SIZE_SU2       0x8000   ///< Groesse des FLASH - Erweiterung des Bereichs fuer Setupdaten (32kByte) 
#define EXTENDED_SIZE_SU     0x4000   ///< Groesse des Erweiterten FLASH - Bereichs fuer Setupdaten (16kByte) 
#define FLASH_SIZE           (FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU)
#define CONFIG_SIZE_SU       0x64     ///< Groesse des Speicherbereichs fuer Grunddaten des Geraets (100Byte)
#define FLASH_KONFIG_START   0x0FC000 ///< Startadresse, Adressoffset fuer alle Zeicherwerte 
#define FLASH_LIST_START     0x0E0000 ///< Startadresse des Listenspeichers, es sind somit 100kByte Gesamtspeicher verfuegbar.
#define FLASH_LIST_START_NEW 0x180000 ///< Startadresse des Listenspeichers, ab 04.00.23 Listengroesse auf 244kByte erhoeht. 
#define FLASH_SIZE_LIST      0x018000 ///< Groesse des Listenspeichers (98kByte) 
#define FLASH_SIZE_LIST_NEW  0x03D000 ///< Groesse des Listenspeichers ab 04.00.23 (ComVersion 5.2) auf 244kByte erhoeht.
#define MAX_FLASH_MEM        0x100000 ///< Groesse des eingesetzten Flashspeichers


/////////////////////////////////////////////////////////////////////
// Formate fuer Zeigerablage

/////////////////////////////////////////////////////////////////////
// Versionsnummern, 
// Zeiger die im EEPROM - Bereich liegen und auf das Flash verweisen.

#define P_VERSION          0x0000 ///< Kommunikationsversionsnummer des Setup. (16bit)
#define P_KONFIG           0x0004 ///< Konfiguration 
#define P_HEADLINE         0x0008 ///< Kopfzeile im Hauptmenue (17 Zeichen + Nullterminie)
//FREE          0x001A - 0x0023

#define P_LIST_STRUCT      0x0024 ///< Startadresse Listenbeschreibung 
#define P_LIST_0           0x0028 ///< Adresse der Daten der Liste Nr. 0 im Flash. Die weiteren Adressen sind nachfolgend gespeichert. z.B. Adresse von Liste Nr. 4  P_LIST_0+4 * sizeof(unsigned int) 
// bis -> 0x0074 Zeiger auf letzte Liste 
#define P_DAT_STRUCT       0x0080 ///< Kopie vonP_DAT_STRUCT Fehler von ATmega zu umgehen 

#define P_FLASH_SIZE_SU    0x0084 ///< Groesse der Flashdaten
#define P_MDE_EK_SIZE      0x0088 ///< Groesse der MDE - Eingabeketten
#define P_ENTRANCE_EK_SIZE 0x008C ///< Groesse der Zutrittskontroll - Eingabeketten
#define P_LIST_KEY_FIELD   0x0090 ///< Fuer alle 20 Moeglichen LSBs je ein Byte fuer ein Schluesselfeld reservieren. 0=Keines zugewiesen, 1..MAX_FELD+1=Feldindex
#define P_AUTO_KEY         0x00A5 ///< Tastenumschaltung
									// Aufbau des Datensatztarray
									// ++++++++++++++++++++++++++
									// 4 Byte Verwaltung (Version etc. Derzeit immer 0)
									// 15 * 4 Byte Datensaetze wobei,
									//      erstes  Byte: Funktion
									//                    0: Nicht gewaehlt
									//                  250: Hauptmenue
									//                  251: Letzte Taste
									//                  252: Letzte Tasten F1, F2
									//                 1..6: Funktionstaste F1 .. F6
									//      zweites Byte: Stunde
									//      drittes Byte: Minute
									//      viertes Byte: Wochentag
									//                    Bit 0 .. Bit 6: Montag .. Sonntag
#define P_SETUP_VERSION    0x00E5 ///< Ab: 03.01.30.00; Es werden in vier Byte die Versionsnummer des Setup mit Buildnumber abgelegt
#define P_DEVICE_TYPE      0x00E9 ///< Ab: 03.01.31.00; Es wird in einem Byte der Geraetetyp abgespeichert, fuer den dieses Setup gespeichert wird. 
#define P_LIST_ACCESS      0x00EA ///< Zeiger auf Strukturen der Zutrittskontrolle: FW 04.01.01
#define P_IC_ACCESS        0x00EE ///< Zeiger auf Strukturen der Zutrittskontrolleingabeketten : FW 04.01.01
#define P_FILE_VERSION     0x00F2 ///< Ab: 04.01.06.09; Es wird eine Dateiversion der Form 0xDF01, 0xDF02 abgelegt
#define P_FREE             0x00F4
// 0x00FF VOLL


/////////////////////////////////////////////////////////////////////
// Strukturdeklarationen
// Es werden immer erst die defines eingefuehrt, welche in den Strukturen
// und allen weiteren evntl. benoetigt werden. Danach folgt die jeweilige Strukturdeklaration.

///////////////////////////////////////
// Struct: StructDatensatzBeschreibung / StructTable / StructTable2
// Beschreibung einer Tabelle

#define MAX_DATENSATZ_BESCHREIBUNG 20 ///< Maximale Anzahl der Datensatzbeschreibungen und auch max. Listenbeschreibungen
#define MAX_FELD                   25 ///< Maximale Anzahl Felder Felder in einem Datensatz.
#define MAX_NAME                   17 ///< Maximale Anzahl Zeichen fuer Namen wie fuer Datensatz/Feld-Namen incl. abschliessende Null
#define MAX_DATENSATZ             241 ///< Maximale Groesse eines Datensatzes (255 - DS_KOPF_CRC)
#define MAX_UNTERPUNKTE           100 ///< Maximale Anzahl Unterpunkte im Baum (Bedienung) 

/** Diese Struktur beschreibt mit ihren Grundangaben eine Tabellendefinition.

  Die Informationen einer Datensatz- sowie Listenbeschreibung, werden in dieser Struktur abgelegt.
  Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
  Es kommt zuerst ein Zeiger - Array einer Laenge von MAX_DATENSATZ_BESCHREIBUNG * unsigned int,
  diese Zeiger verweisen jeweils auf eine der Maximal MAX_DATENSATZ_BESCHREIBUNG moeglichen.
*/
typedef struct StructDatensatzBeschreibung {

	unsigned char Felder; ///< Gibt die Anzahl der Tabellenfelder an.
	char Name[MAX_NAME];  ///< Gibt den verwendeten Tabellenname an. 
	unsigned int P;	  ///< Gibt den Zeiger auf das Array der Zeiger fuer die Feldbeschreibungen (Typ "TDatensatzFeld") an.

} TDatensatzBeschreibung;

/** Diese Struktur ist eine Uebergangsstruktur von StructDatensatzBeschreibung auf StructTable2. 

  Naehere Informationen siehe StructTable2.
*/
typedef struct StructTable {

	unsigned char count;  ///< Anzahl Felder im Datensatz 
	char name[MAX_NAME];  ///< Name der Datensatzbeschreibung
	unsigned int ptr;    ///< Zeiger auf Array fuer Zeiger auf Feldstrukturen
	unsigned char num;    ///< Nummer der Tabelle im System
	unsigned char key;    ///< Schluesselspalte
	unsigned char sort;   ///< Sortierspalte
	unsigned short flags; ///< Flags

} STable;

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellendefinition.

  Diese Struktur ersetzt die nicht mehr ausreichende StructDatensatzBeschreibung.
  Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/

#define TF_SORT   0x0001 // Tabelle soll lexiographisch sortiert werden und -sort- gibt nullindex der Spalte an
#define TF_SORTED 0x0002 // Listendaten sind lexiographisch nach spaltenindex -sort- sortiert.

typedef struct StructTable2 {

	unsigned char num;    ///< Nummer der Tabelle im System
	char name[32];        ///< Name der Tabelle.
	unsigned char count;  ///< Anzahl Tabellenfelder
	unsigned char key;    ///< Schluesselspalte
	unsigned char sort;   ///< Sortierspalte
	unsigned short flags; ///< Flags

} STable2;


///////////////////////////////////////
// Struct: StructDatensatzFeld / StructTableField / StructTable2Field
// Beschreibung eines Tabellenfeldes 

//	Feldtypen mit Laengenangabe
#define TF_ULONG                        1 ///< StructDatensatzFeld.Typ: Unsigned int (4 Byte)
#define TF_ULONG_LEN                    4 ///< StructDatensatzFeld.Cnt: Unsigned int (4 Byte)
#define TF_DT                           2 ///< StructDatensatzFeld.Typ: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_DT_LEN                       7 ///< StructDatensatzFeld.Cnt: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_NUM                          3 ///< StructDatensatzFeld.Typ: ASCII (nur Ziffern)
#define TF_A_NUM                        4 ///< StructDatensatzFeld.Typ: ASCII
#define MAX_LENGTH_FELD                21 ///< StructDatensatzFeld.Cnt: Maximal moegliche Feldlaenge. (ASCII incl. Null-term.)
#define TF_DATE                         5 ///< StructDatensatzFeld.Typ: Date (4 Byte; Y100, Y, M, D)
#define TF_DATE_LEN                     4 ///< StructDatensatzFeld.Cnt: Date (4 Byte; Y100, Y, M, D)
#define TF_TIME                         6 ///< StructDatensatzFeld.Typ: Time (3 Byte; h, m, s)
#define TF_TIME_LEN                     3 ///< StructDatensatzFeld.Cnt: Time (3 Byte; h, m, s)
#define TF_FINGERTEMPLATE_DIN           7 ///< StructDatensatzFeld.Typ: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_DIN_LEN     161 ///< StructDatensatzFeld.Cnt: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_IDENCOM       8 ///< StructDatensatzFeld.Typ: Fingertemplate (216 Byte; Idencom Compact)
#define TF_FINGERTEMPLATE_IDENCOM_LEN 216 ///< StructDatensatzFeld.Cnt: Fingertemplate (216 Byte; Idencom Compact)

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.


  Die Informationen einer Datensatzfeld- sowie Listenfeldbeschreibung, werden in dieser Struktur abgelegt.
  Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
  Es kommt zuerst ein Zeiger - Array einer Laenge von StructDatensatzBeschreibung.Felder * unsigned int,
  diese Zeiger verweisen jeweils auf eine der StructDatensatzBeschreibung.Felder vorhandenen.
*/
typedef struct StructDatensatzFeld {

	unsigned char Typ;            ///< Feldtyp 
	unsigned char Cnt;            ///< Feldlaenge inc. anbschlieende Null bei Strings 
	unsigned char Name[MAX_NAME]; ///< Feldname

} TDatensatzFeld;

/** Diese Struktur ist eine Uebergangsstruktur von StructDatensatzFeld auf StructTable2Field.

  Naeheres siehe StructTable2Field.
*/
typedef struct StructTableField {

	unsigned char type;   ///< Feldtyp 
	unsigned char size;   ///< Feldlaenge inc. anbschlieszende Null bei Strings 
	char name[MAX_NAME];  ///< Feldname
	unsigned char num;    ///< Nummer des Tabellenfeldes
	unsigned short flags; ///< Flags
	char format[32];      ///< Formatstring fuer Feldwertmoeglichkeit

} STableField;

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.

  Diese Struktur ersetzt die nicht mehr ausreichende StructDatensatzFeld.
  Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/
typedef struct StructTable2Field {

	unsigned char num;    ///< Nummer des Tabellenfeldes
	char name[32];        ///< Feldname
	unsigned char type;   ///< Feldtyp
	unsigned char size;   ///< Feldlaenge inc. anbschlieszende Null bei Strings 
	unsigned short flags; ///< Flags
	char format[32];      ///< Formatstring fuer Feldwertmoeglichkeit

} STable2Field;


/////////////////////////////////////////////////////////////////////
// Bediensteuerung 
// Hautpmenue mit Menueeintraegen, Eingabeketten und Feldern

///////////////////////////////////////
// Struct: StructBedien

// Typen der Bedienung 
#define TB_MENUE		1 ///< StructBedien.Typ: Folgestruktur ist von Typ: StructMenue
#define TB_DATENSATZ	2 ///< StructBedien.Typ: Folgestruktur ist von Typ: StructDatensatzEinlesen
#define TB_SYSTEM		3 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_KONFIG_ROOT	4 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_LISTE		5 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_CLR_LAST		6 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert

/** Informationsstruktur zur Datensteuerung der Bedienung.

  In der Geraetebedienung kann von festen Informationskonstrukten gesprochen werden, an stellen wo
  meherer Wege eingeschlagen werden koennten, kommt diese Struktur zum Einsatz. Es kann auf die
  Bedienungswurzel z. B. sowohl eine Menuestruktur oder ein anderer Typ folgen. In einem Menue selbst sind
  hingegen nur Menueeintraege erlaubt. In diesen jedoch wieder verschiednene Typen wie Eingabeketten etc.
  Um Mehrdeutigkeiten aufzuloesen kommt somit vor jeder StructMenue und StructDatensatzEinlesen eine StructBedien.
*/
typedef struct StructBedien {

	unsigned char Typ; ///< Typ der Folgestruktur 
	unsigned char d1;  ///< Reserve 
	unsigned char d2;  ///< Reserve
	unsigned int P;   ///< Zeiger auf Folgestruktur, welche durch "Typ" angegeben wird. 

} TBedien;


///////////////////////////////////////
// Struct: StructMenue

/** Diese Struktur beinhaltet Informationen zum Hauptmenue / Untermenues.


*/
typedef struct StructMenue {

	unsigned char Flags;      ///< Flags wie Zutrittskontrollabfrage aktiviert usw..
	                          ///< Bit 0: Flag fuer die zweispaltige Anzeige beim AE-MasterIV. 
                              ///< Bit 1: Menauswahl per Barcode zulassen.
                              ///< Bit 2: Menauswahl per Tranponder zulassen.
	unsigned char EintragCnt; ///< Anzahl Eintraege im Menue 
	char Name1[MAX_NAME];     ///< Ueberschrift1 des Menues 
	char Name2[MAX_NAME];     ///< Ueberschrift2 des Menues
	/*albr< 11.05.09*/
	unsigned char K1Txt;	  ///< Text in Kopfzeile 1 Bit 0...3 Nummer-Globale-Variable nur fr das Hauptmen gltig sonst 254
	unsigned char K2Txt;	  ///< Text in Kopfzeile 2 Bit 0...3 Nummer-Globale-Variable nur fr das Hauptmen gltig sonst 254
	/*albr>*/
	unsigned char GlobVar;    ///< GlobVar in die der Wert des Barcodes oder Transponders zur Auswahl des Meneintrags kopiert werden soll.
  unsigned char EscV;       ///< Verhalten bei ESC-Taste, Sprungziel (Bedeutung siehe StructDatensatzEinlesen)
	unsigned char Dummy[4];   ///< Reserve
	unsigned int P;          ///< Zeiger auf Liste der Eintraege 

} TBMenue;


///////////////////////////////////////
// Struct: StructKonfigRoot

// Ein Varhalten 
#define EVERH_AKTUELL 0 ///< StructKonfigRoot.AutoAus: Nach Ein auf der aktuellen Pos vor Aus bleiben 
#define EVERH_MENUE   1 ///< StructKonfigRoot.AutoAus: Nach Einschalten zurueck in das Menue
                        ///<                           Eintrag aus Hauptmenue starten

/** Diese Struktur beinhaltet Informationen zum Haupteintrag der Bedienung.

  Dieser Struktur folgt ein Zeigerarray mit EintragCnt (* sizeof(unsigned int)) Zeigern, die auf folgende
  StructMenue verweisen.
*/
typedef struct StructKonfigRoot {

	unsigned short AutoAus;     ///< Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfolgt
                                ///< 0 -> kein Automatische Abschaltung 
	unsigned char EintragCnt;   ///< Anzahl Eintraege in KonfigRoot 
	unsigned char EinVerhalten; ///< Verhalten nach Einschalten
	unsigned char Dummy[8];     ///< Reserve 
	unsigned int P;            ///< Zeiger auf Zeigerarray mit "EintragCnt" Eintraegen 

} TBKonfigRoot;


///////////////////////////////////////
// Struct: StructMenueEintrag

// Flags fuer Menueeintrag
#define BDME_MDE_0 0x00 ///< StructMenueEintrag.Flags: Normal
#define BDME_MDE_1 0x01 ///< StructMenueEintrag.Flags: Anmeldung
#define BDME_MDE_2 0x02 ///< StructMenueEintrag.Flags: Abmeldung
#define BDME_MDE_3 0x03 ///< StructMenueEintrag.Flags: Taste klemmen
#define BDME_MDE_4 0x04 ///< StructMenueEintrag.Flags: Taste klemmen und kummulieren
#define BDME_MDE_5 0x05 ///< StructMenueEintrag.Flags: Buchung
#define BDME_MDE_ONLY_BY_ORDER 0x80 ///< StructMenueEintrag.Flags: Ausfuehrung ist nur bei angemeldetem Auftrag moeglich.

/** Diese Struktur enthaelt Informationen zu einem Menueeintrag.

  Dieser Struktur folgt ein Zeigerarray mit EintragCnt (* sizeof(unsigned int)) Zeigern, die auf folgende
  StructDatensatzEinlesen oder StructMenue verweisen.
*/
typedef struct StructMenueEintrag {

	char Name[MAX_NAME];      ///< Menue Text 
	unsigned char EintragCnt; ///< Anzahl Eintraege in KonfigRoot 
	unsigned char PassL;      ///< Passwort Low-Byte 
	unsigned char PassH;      ///< Passwort High-Byte  0 -> kein Passwort 
	unsigned char Flags;      ///< Parameter fuer MDE 
	unsigned char machine;    ///< Zugeordnete Maschine bei Taste Klemmen
	unsigned char Dummy[4];   ///< Reserve 
	unsigned int P;          ///< Zeiger auf Aktion nach Auswahl des Menuepunktes 

} TBMenueEintrag;


///////////////////////////////////////
// Struct: StructDatensatzEinlesen

// Einlesemetoden fuer Datensaetze 
#define TBDA_NORM 0 ///< StructDatensatzEinlesen.Typ: Standard 

// Flags Datensatz einlesen 
#define BDEF_W_UEBER			      0x01 ///< StructDatensatzEinlesen.Flags: Bei Schleifen bereits eingegebene Werte als 
                                     ///<                                Vorgabe fuer Eingabe uebernehmen 
#define BDEF_NICHT_SPEICHERN	  0x02 ///< StructDatensatzEinlesen.Flags: Datensatz nicht speichern (nur Globale Variable) 
#define BDEF_ZUSATZ_RETURN		  0x04 ///< StructDatensatzEinlesen.Flags: Abfrage "Speichern ?" am Ende der Eingabekette 
#define BDEF_RUECKSPRUNG_MENUE	0x08 ///< StructDatensatzEinlesen.Flags: Nach Einlesen eines Datensatzes Ruecksprungmenue anzeigen 
#define BDEF_ENTRANCE_ACCESS    0x10 ///< StructDatensatzEinlesen.Flags: Zutrittskontrolle vor Abarbeitung der Eingabekette durchfuehren.
#define BDEF_RELAIS             0x20 ///< StructDatensatzEinlesen.Flags: Relais fuer bestimmte Zeit schalten.
#define BDEF_RECORD_DOUBLE      0x40 ///< StructDatensatzEinlesen.Flags: Eingabekette vor evntl. Eingabe von Werten ausfuehren, danach "Normale" Abarbeitung. 
#define BDEF_NO_SERVER_ONLINE   0x80 ///< StructDatensatzEinlesen.Flags: Im Modus "Server online" wird das Verhalten, dass auf die Antwort und Sendmessage gewaretet wird, fuer diesen Datensatz deaktiviert.

/** Diese Struktur enthaelt Informationen zu einer Eingabekette.

  Dieser Struktur folgt ein Zeigerarray mit Felder (* sizeof(unsigned int)) Zeigern die auf folgende 
  StructFeldEinlesen verweisen. Ihr selbst ist immer eine StructBedien vorangestellt.
*/
typedef struct StructDatensatzEinlesen {

	unsigned char DsNum;         ///< Nummer der zugeordneten Datensatzbeschreibung (ab 0, 255 = Keine Zuordnung) 
	unsigned char Felder;        ///< Anzahl Felder
	unsigned char CrV;           ///< Verhalten bei Enter-Taste, Sprungziel 
                    //Wert	   Bedeutung
                    //---------------------
                    //0        Uebergeordnetes U-Menue
                    //1        Hauptmenue (war mal ausschalten)
                    //253      Reserviert. Naechstes Eingabekettenfeld.
                    //254      Hauptmenue
                    //255      Naechste E-Kette bzw. U-Menue
                    //2-111    (1-109)-tes E-Feld der Eingabekette
                    //112-127  (1-15)-te FTaste des Hauptmenue
                    //200-229  (1-30)-te E-Kette bzw. U-Menue im gleichen Menueeintrag.
                    //230-252  Reserviert. Wird zu 0 konvertiert, wenn Bedeutung unbekannt.
	char Name[MAX_NAME];         ///< Ueberschrift 
	unsigned char Typ;           ///< Bedien-Typ des Einlesens 
	unsigned char Flags;         ///< Diverse Flags Definiert als BDEF_.... 
	unsigned char K1Txt;         ///< Text in Feld K2 Bit 0...3 Nummer-Globale-Variable 
                                 ///<                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. 
	unsigned char EscV;          ///< Verhalten bei ESC-Taste, Sprungziel
                    //Wert	   Bedeutung
                    //---------------------
                    //0        Uebergeordnetes U-Menue
                    //1        Hauptmenue (war mal ausschalten)
                    //253      Reserviert. Naechstes Eingabekettenfeld.
                    //254      Hauptmenue
                    //255      Naechste E-Kette bzw. U-Menue
                    //2-111    (1-109)-tes E-Feld der Eingabekette
                    //112-127  (1-15)-te FTaste des Hauptmenue
                    //200-229  (1-30)-te E-Kette bzw. U-Menue im gleichen Menueeintrag.
                    //230-252  Reserviert. Wird zu 0 konvertiert, wenn Bedeutung unbekannt.
	unsigned char dummy;         ///< war mal Flags fuer Master - Slave - Felder.
	unsigned char ucRelaisNum;   ///< Nummer des anzusprechenden Relais.
	unsigned short usRelaisTime; ///< Zeit fuer die das ausgewaehlte Relais schliesst.
	unsigned char Dummy;         ///< Reserve 
	unsigned int P;             ///< Zeiger auf Liste der Strukturen fuer Einlesen der Felder 
                                 ///< (Anzahl und Reihenfolge mu mit Datensatzbeschreibung uebereinstimmen) 
} TBDatensatzEinlesen;


///////////////////////////////////////
// Struct: StructExBestaetigung

/** Diese Struktur enthaelt die Textzeilen eines Bestaetigungsfeldes.

  Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
typedef struct StructExBestaetigung {
	char Text1[MAX_NAME]; ///< Hinweistext der ersten Zeile.
	char Text2[MAX_NAME]; ///< Hinweistext der zweiten Zeile.
	char Text3[MAX_NAME]; ///< Hinweistext der dritten Zeile.
} TBExBestaetigung; // 51 Byte


///////////////////////////////////////
// Struct: StructCompare

#define FECT_NONE            0x00 ///< StructCompare.type: Kein Sprung
#define FECT_FORMAT_STRING   0x01 ///< StructCompare.type: Sprung mit Formatstring konstant.
#define FECT_FORMAT_GV       0x02 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen
#define FECT_FORMAT_STRING_2 0x03 ///< StructCompare.type: Sprung mit Formatstring konstant. FW >= 04.01.04
#define FECT_FORMAT_GV_2     0x04 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen. FW >= 04.01.04

/** Diese Struktur dient mit Ihren Informationen der Angabe eines Sprunges in einer Eingabekette.

  Diese Struktur kommt als Erweiterung der Struktur StructExNumFeld zur Anwendung.
*/
typedef struct StructCompare {

	unsigned char type;            ///< Typ fuer die Vergleichsart; Formatstring || GV
	unsigned char formatGV;        ///< Wenn type GV: Bit 0..3: Arraynummer der GV, anderen Bits fuer Comboboxindex
	char formatString[MAX_NAME];   ///< Wenn type Formatstring: Der Formatstring.
	unsigned char negate;          ///< Verzweigen wenn Vergleich NICHT zutraf.
	unsigned char jumpDestination; ///< Sprungziel, als Feldindex (0..n), von Eingabekette.

} TBCompare; // 21 Byte


///////////////////////////////////////
// Struct: StructTransponder

// Flags fuer TP TBTransponder.tbModeX
#define BDTP_ACTIVE     0x80 ///< StructTransponder.tpMode: Einstellung fuer Transponder aktiviert.
//  lesen / schreiben
#define BDTP_WRITE      0x40 ///< StructTransponder.tpMode: Der Wert wird ins Segment geschrieben, ansonsten gelesen.

#define BDTP_DEC        0x00 ///< StructTransponder.tpMode: Behandlung der Daten als Dezimalwert.
#define BDTP_HEX        0x10 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert.
#define BDTP_ASCII      0x20 ///< StructTransponder.tpMode: Behandlung der Daten als ASCII - Zeichenfolge.
#define BDTP_HEX_TURNED 0x30 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert aber umgekehrte Bytefolge.

// Untere 4bit sind Anzahl feste stellen

/** Diese Struktur dient bei Angabe eines Hitagtransponders, der zu verwendenden Segmente, Ablageformate sowie Lnge.

  Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
  um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
typedef struct StructTransponder {

	unsigned char tpSegment1;  ///< Gibt das erste Segment an, aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
	unsigned char tpMode1;     ///< Gibt den Modus des Wertes an. Oberes Nibble beinhaltet oben definierte Flags, unteres die feste Laengenangabe. 
	unsigned char tpSegment2;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein zweites Segment.
	unsigned char tpMode2;     ///< siehe Beschreibung zu tpMode1, hier fuer einen zweiten Mode.
	unsigned char tpSegment3;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein drittes Segment.
	unsigned char tpMode3;     ///< siehe Beschreibung zu tpMode1, hier fuer einen dritten Mode.

} TBTransponder; // 6 Byte

/** Diese Struktur dient bei Angabe eines Mifaretransponders, der zu verwendende Sektor, Ablageformat sowie Auswahl.

  Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
  um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
typedef struct StructTransponderMifare {

	unsigned char sectorBlock;  ///< Gibt im oberen Nibble den Sektor, im unteren den Block an, 
	                            ///< aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
	unsigned char mode;         ///< Oberes Nibble beinhaltet oben definierte Flags fuer das Datenformat, 
	                            ///< unteres die feste Laengenangabe. Bei Hex-Format ist der Wert mal 2 zu nehmen.
	unsigned char cut;          ///< Gibt im oberen Nibble die Startposition des ersten zu lesenden Bytes der 16 moeglichen an.
	                            ///< Das untere Nibble ist frei.
	unsigned char passwordMode; ///< Gibt den zu verwendenden Passwortbereich an.
	unsigned char securityLevel;///< Gibt den zu verwendenden Security-Level des Mifare-Plus ausweises an.
	unsigned char tmp2;         ///< Frei.

} STransponderMifare; // 6 Byte

/** Diese Struktur dient bei Angabe eines HID ProxKey, HID iClass Transponders, der zu verwendende Ausweistyp, Ablageformat sowie Auswahl.

  Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
  um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
typedef struct StructTransponderHIDiClass {

	unsigned char type;         ///< Gibt den zu verwendenden Ausweistyp an.
	                            ///< 0 - H10301
	                            ///< 1 - H10302
	                            ///< 2 - H10304
	                            ///< 3 - Freies Datenformat
	unsigned char format;       ///< Ablageformat.
	                            ///< 0 - Dezimal
		                        ///< 1 - Hexadezimal
		                        ///< 2 - Reserviert fr ASCII nicht untersttzt!
		                        ///< 3 - Hexadezimal gedreht
	unsigned char length;       ///< Ablage feste Laengenangabe.
	                            ///< 0 - Keine feste Lnge
	                            ///< n - Anzahl Zeichen ggf. mit Vornullen auf die angegebene Lnge bringen.
	unsigned char tmp1;         ///< Frei.
	unsigned char tmp2;         ///< Frei.
	unsigned char tmp3;         ///< Frei.

} STransponderHIDiClass; // 6 Byte

/** Diese Struktur dient bei Angabe eines Legic Advant Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
typedef struct StructTransponderLegicAdvant {

	// Die ersten 6 Byte muessen mit STransponderLegicAdvantEKF identisch sein.
	unsigned char segment;      ///< Gibt das zu verwendende Segment an. Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
	unsigned short pos;         ///< Position des zuerst gelesenen Datenbyte.
	unsigned char length;       ///< Anzahl zu lesender Datenbyte.
	unsigned char format;       ///< Gibt das abzulegende Werteformat an.
	                            ///< 0 - Dezimal
	                            ///< 1 - Hexadezimal
	                            ///< 2 - ASCII
	                            ///< 3 - Hexadezimal gedreht
	unsigned char formatLength; ///< Gibt die abzulegende Werteformatlaenge an.
	                            ///< 0 - Keine feste Laengenangabe
	                            ///< n - Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
	unsigned char typeFlags;    ///< Enthlt die gewuenschten Ausweistypen und zusaetzliche Flags.
	                            ///< Bit 8 und 7 (Maske 0xC0) beinhaltet den Ausweistyp
	                            ///<    0 - Nur Advant Ausweise
	                            ///<    1 - Nur Prime Ausweise
	                            ///<    2 - Advant und Prime Ausweise
	                            ///<    3 - Frei
	                            ///< Bit 6 und 5 (Maske 0x30) CRC Pruefung
	                            ///<    0 - Keine Pruefung
	                            ///<    1 - 8Bit
	                            ///<    2 - 16Bit
	                            ///<    3 - Frei
	                            ///< Bit 4. Bei gesetztem Bit ist Access Segment Definition aktiv.
	                            ///< Bit 3. Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
	                            ///< Bit 2. Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
	                            ///< Bit 1. Bei gesetztem Bit sind Seriennummern von Prime Ausweisen nach dem Advant Format zu liefern.
	unsigned char valType;      ///< Zu Lesender Wertetyp.
	                            ///< 0 - Seriennummer
	                            ///< 1 - Freie Datenauswahl
	                            ///< 2 - Kaba group Header von Prime Ausweisen lesen.
	                            ///< 3 - Kaba group Header mittels Suche finden.
	                            ///< ... - Frei
	unsigned short crcAddress;  ///< Position des CRC
	unsigned char kghSearch[13];///< Zu suchende Bytefolge fuer den Kaba group header
	unsigned char kghSearchLength; ///< Laenge der zu suchenden Bytefolge des Kaba group header
	unsigned char flags;        ///< Enthaelt zusaetzliche Flags
	                            ///< Bit 1. Bei gesetztem Bit wird nach KGH gesucht. Nur bei Ausweisen vom Typ Legic Prime.
	                            ///< Bit 2 - 8. Frei.
	unsigned char tmp[5];       ///< Reserve fr Erweiterungen

} STransponderLegicAdvant; // 30 Byte

///////////////////////////////////////
// Struct: StructExNumFeld

// Quellen fuer Eingabe von Daten (Maske in Var. Flag) 
#define NFF_TASTATUR	0x80 ///< StructExNumFeld.Flags: Werteingabe ueber Tastatur moeglich.
#define NFF_BARCODE		0x40 ///< StructExNumFeld.Flags: Werteingabe ueber Barcode moeglich.
#define NFF_TRANSPONDER 0x20 ///< StructExNumFeld.Flags: Werteingabe ueber Transponder moeglich.
#define NFF_NO_RET		0x01 ///< StructExNumFeld.Flags: CR nach Barcodeeingabe notwendig.
#define NFF_ANZEIGEN	0x0E ///< StructExNumFeld.Flags: 3 Bit Maske fuer 8 Werte der Anzeigedauer, wenn kein Return notwendig. 
#define NFF_BUZZER      0x10 ///< StructExNumFeld.Flags: Kein Aktustisches Signal zur Besaetigung

/** Diese Struktur dient in Eingabekettenfeldern bei allen Einlesemethoden ausser Liste und Bestaetigung, 
    zur Angabe weiterer Informationen.

  Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
typedef struct StructExNumFeld {

	unsigned char Flags;       ///< Flags z.B. Einlesen ueber Barcode erlaubt 
	TBTransponder tag;         ///< Transpondereinlesen bei Hitag, bei diesem Feld abaendern. Abweichend von der globalen Einstellung.
	unsigned char fingerprintParam; ///< Parametervariable fuer Fingerprint
                                    ///< Bit 1+2: Destination: (0)Fingerprint modul, (1)Ausweis
                                    ///< Bit 3+4: Delete Method: (0)Fingertemplate, (1)PID
                                    ///< Bit 5+6: Fingertemplate: (0)Nr 1, (1)Nr 2
	unsigned char fingerprintPID;   ///< Index der GV fuer PID-Quelle
	unsigned char operation;        ///< Mathematische oder Logische Operation ausfhren (Fuer TBFE_OPERATION)
	                                ///< Aufbau: Bit(7,6) = Linker Operant, Bit(5,4,3,2) = Operation, Bit(1,0) = Rechter Operant
	                                ///< Operant: 0 = Konst. 1 = Feld, 2 = GV
	                                ///< Operation: 0 = +, 1 = -, 2 = Not
	unsigned char relayFlagsNum;    ///< Flags und verwendete Relaisnummer (Fuer TBFE_SWITCH_RELAY, TBFE_SWITCH_LED, TBFE_SWITCH_BUZZER)
	                                ///< Oberen Nibble bei Relais und Buzzer: 
	                                ///< Bit(3,2) = Aktion vor Zeitablauf; Bit (1,0) = Aktion nach Zeitablauf
	                                ///< Aktion: 0 = Ausschalten, 1 = Einschalten, 2 = Umschalten, 3 = Keine
	                                ///< Oberes Nibble bei LED:
	                                ///< Aktion: 0 = beide aus, 1 rot an grn aus, etc.
	                                ///< Unteres Nibble: Zu schaltende Relaisnummer oder bei LEDs die Maschinennummer.
	                                ///< Unteres Nibble bei Buzzer: Lautstaerke in 10er Schritten fuer % der Lautstaerke.
	unsigned short relayTime;       ///< Zeit fuer Schaltdauer in Sekunden.
	char KonstAscii[MAX_NAME]; ///< Wert der Konstante bei Einlesemethode "Konstante uebernehmen".
	TBCompare compare;         ///< Sprung in einer Eingabekette definieren.
	
} TBExNumFeld; // 30 Byte


///////////////////////////////////////
// Struct: StructExListAuswahl

#define BDLB_SEL_BARCODE		0x01 ///< StructExListAuswahl.SelBarcode: Selection in Liste mit Barcode freigeben 
#define BDLB_BESTAETIGEN		0x02 ///< StructExListAuswahl.SelBarcode: Ausgewaehlten Eintrag bestaetigen mit Ret 
#define BDLB_AUSBLENDEN			0x04 ///< StructExListAuswahl.SelBarcode: Automatisches Ausblenden der Liste nach n Sekunden. 

#define BDLB_SEL_TRANSPONDER    0x01 ///< StructExListAuswahl.Flags: Selection in Liste mit Transponder freigeben 
#define BDLB_SEL_TASTATUR       0x02 ///< StructExListAuswahl.Flags: Selection in Liste mit Tastatur freigeben 

/** Diese Struktur dient in Eingabekettenfeldern bei der Einlesemethode Liste, 
    zur Angabe von weiteren Informationen.

  Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
typedef struct StructExListAuswahl {

	unsigned char Sel;	        ///< Flag Listenauswahl mit Selection 
	unsigned char SelDFeld;     ///< Feld im Datensatz fuer Selection (nur wenn "Sel" = TRUE) 
	unsigned char SelLFeld;     ///< Feld in der Liste nach dem selectiet wird (nur wenn "Sel" = TRUE) 
	unsigned char List;         ///< Nummer der Liste 
	unsigned char AuswahlFeld;  ///< Feld in der Liste nach dem der Benutzer auswaehlt 
	unsigned char DatFeld;      ///< Feld in der Liste das im Datensatz gespeichert wird 
	unsigned char SelBarcode;   ///< Die unteren drei Bit sind Flags, die oberen der Index der ausgewaehlten Anzeigedauer.
	unsigned char Flags;        ///< Eingabequellen: Transponder, Tastatur
	unsigned char NoValueCheck; ///< Wert nicht in Liste
	unsigned char tempa[5];     ///< Tempfeld um hinter TBExNumFeld zu kommen.
	unsigned char doubleColumn; ///< War mal: MapFKey, Die ersten zehn Listeneintraege auf Funktionstasten legen.
	                            ///< Ist Jetzt: Zweispaltige Anzeige beim AE-MasterIV
	unsigned char SelDFeld2;    ///< wie SelDFeld
	unsigned char SelLFeld2;    ///< wie SelLFeld
	unsigned char SelNoEntry;   ///< Verhalten wenn kein Eintrag vorhanden.
	unsigned char usedFontSize; ///< Zu verwendente Schriftgroesze bei Listenanzeige
	unsigned char temp[11];     ///< Reserve.
	TBCompare compare;

} TBExListAuswahl; // 15 Byte


///////////////////////////////////////
// Struct: StructFeldEinlesen

// Einlesemethoden fuer Felder 
#define TBFE_NORM                        1 ///< Standardmethode.
#define TBFE_GET_TIME                    2 ///< Aktuelles Datum-Uhrzeit uebernehmen.
#define TBFE_LIST_SELECT_GET             3 ///< Aus Liste auswaehlen um Wert von Liste zu uebernehmen.
#define TBFE_KONSTANTE                   4 ///< Konstante in den Datensatz schreiben 
#define TBFE_BESTAETIGUNG                5 ///< Bestaetigung abfragen (fuellt kein Datenfeld).
#define TBFE_COPY_GV_TO_FIELD            6 ///< Globale Variable in Feld kopieren.
#define TBFE_SERIENNUMMER                7 ///< Seriennummer uebernehmen 
#define TBFE_ANALOG_WERT                 8 ///< Analogen Messwert in Feld uebernehmen.
#define TBFE_DIGITAL_WERT                9 ///< Digitalen Messwert in Feld uebernehmen.
#define TBFE_ZAEHLER_WERT               10 ///< Zaehlerstand in Feld uebernehmen.
#define TBFE_ENTRANCE_PNR               11 ///< Eingelesene Personalnummer uebernehmen.
#define TBFE_ENTRANCE_STATUS            12 ///< Status von Zutritt uebernehmen.
#define TBFE_FUNC                       13 ///< Funktionswert uebernehmen.
#define TBFE_SPS_WERT                   14 ///< Wert von SPS uebernehmen.
#define TBFE_GPRS_ALIVE_COUNTER         15 ///< Aufgelaufene Alivedatensaetze (Zaehler) uebernehmen.
#define TBFE_FIRMWARE_VERSION           16 ///< Firmware-Version uebernhemen.
#define TBFE_ONLINE_STATUS              17 ///< String "online" oder "offline" als -Server Online- Zustand uebernehmen
#define TBFE_ENTRANCE_ZM_NUM            18 ///< Zutrittsmasternummer uebernehmen.
#define TBFE_ENTRANCE_READER_NUM        19 ///< Lesernummer uebernehmen.
#define TBFE_SUMMER_WINTER_STATUS       20 ///< (S) fuer Sommerzeit oder (W) fuer Winterzeit uebernehmen.
#define TBFE_GPS_RECORD_FIX             21 ///< GPS - Daten uebernehmen. Feste Vorgabe des Aufbaus
#define TBFE_ENTRANCE2_IDENTIFICATION   22 ///< Zutrittspruefung durchfuehren. 
#define TBFE_COPY_FIELD_TO_GV           23 ///< Feldwert in Globale Variable kopieren.
#define TBFE_FINGERPRINT_SCANN          24 ///< Finger scannen und Fingertemplate liefern
#define TBFE_FINGERPRINT_CARD           25 ///< Fingertemplate(s) von Karte lesen.
#define TBFE_FINGERPRINT_TEACH          26 ///< Finger einlernen
#define TBFE_FINGERPRINT_IDENTIFICATION 27 ///< Identifikation durchfuehren
#define TBFE_FINGERPRINT_VERIFICATION   28 ///< Verifikation durchfuehren
#define TBFE_FINGERPRINT_DELETE         29 ///< Fingertemplate loeschen
#define TBFE_TIMEBOY_COPY_GV_TO_FIELD   30 ///< Globale Variable von Timeboy uebernehmen
#define TBFE_ANALOG_STATUS              31 ///< Status eines Analogen Eingangs uebernehmen
#define TBFE_LIST_SELECT_SET            32 ///< Aus Liste auswaehlen um Wert in Liste zu uebernehmen.
#define TBFE_OPERATION                  33 ///< Mathematisch oder Logische Operation ausfuehren.
#define TBFE_TIMEBOY_COPY_FIELD_TO_GV   34 ///< Feldwert in Globale Variable des Timeboy uebernehmen.
#define TBFE_SWITCH_RELAY               35 ///< Relais schalten
#define TBFE_GPS_RECORD_VARIABLE        36 ///< GPS - Daten uebernehmen. Variable Auswahl aus Daten.
#define TBFE_TIMER                      37 ///< Timer starten/stoppen
#define TBFE_TRANSPONDER_WRITE          38 ///< Transponderwert auf Karte schreiben
#define TBFE_SWITCH_LED                 39 ///< LED einer Maschine schalten
#define TBFE_GPS_DISTANCE               40 ///< Zurckgelegte Entfernung in m bernehmen.
#define TBFE_MOBILE_PARAMETER           41 ///< Parameter des Mobilfunks bernehmen. Ordsbestimmung.
#define TBFE_SWITCH_BUZZER              42 ///< Buzzer schalten
#define TBFE_KONSTANTE_TYPE_DEVICE_NAME 43 ///< Geraetetypname fuer welches dieses Setup ist, als Konstante uebernehmen.

#define TBFE_MAX                        44 ///< Maximale Anzahl Funktionen fuer Arraygroesse

#define TBFE_DEBUG                     255 ///< Einlesemethode fuer interne Zwecke

// FeldEinlesenFlag
#define FEF_APPEND_TO_FIELD 0x01 ///< StructFeldEinlesen.flags: Der ermittelte Wert an den Feldwert anfuegen.
#define FEF_APPEND_TO_GV    0x02 ///< StructFeldEinlesen.flags: Der ermittelte Wert an die globale Variable anfuegen.

// Verhalten am Ende einer Eingabekette oder bei Eingabekettenfeld. 
#define EV_UPPER_MENUE    0 ///< Uebergeordnetes Untermenue 
#define EV_NEXT_FIELD   253 ///< Naechstes Eingabekettenfeld
#define EV_MAIN_MENUE   254 ///< Hauptmenue
#define EV_NEXT_SIBLING 255 ///< Naechste Eingabekette bzw. U-Menue. 

/** Diese Struktur enthaelt Informationen zu einem Eingabekettenfeld.

  Der enthaltene union-Teil wird abhaengig von dem verwendeten Typ genutzt.
*/
typedef struct StructFeldEinlesen {

	unsigned char Typ;         ///< Typ der Einlesemetode "TBFE..." 
	unsigned char DFNum;       ///< Nummer des Datensatzfeldes (ab 0) 
	unsigned char EscV;        ///< Verhalten bei ESC-Taste (Bedeutung siehe StructDatensatzEinlesen)
	char Name[MAX_NAME];       ///< Ueberschrift 
	unsigned char K2Txt;       ///< Text in Feld K2 Bit 0..3 Nummer-Globale-Variable 
							   ///< Bit 4..7 = 0 -> Konstanter Text, 4..7 > 0 -> Glob.-Var. 
	unsigned char GlobVar;     ///< Feld-Wert in Globale Var. kopieren (Aufbau wie K2Txt)
	/*albr< 24.06.09*/
	/*unsigned char flags;*/       ///< FEF_... 
	unsigned char SysVar;      ///< Bit 8 = 1 -> Wert einer Systemvariablen wird in Zeile 4 der Anzeige angezeigt
							   ///							das Offset, welcher Typ angezeit wird steht in K2Txt
							   ///							die Adresse der Systemvariable steht in MessVar1
							   ///  Bit 7 = 1 -> Wert einer Systemvariablen wird geschrieben
							   ///							das Offset, welcher Typ angezeit wird steht in GlobVar
							   ///							die Adresse der Systemvariable steht in MessVar2
	/*albr>*/
	unsigned char BarcodeVon;  ///< Bereich aus Barcode ausschneiden 1 -> erstes Zeichen 0 -> deaktiviert 
	unsigned char BarcodeBis;  ///< 
	unsigned char InputTypCnt; ///< Bei Methode Normal Feldlaenge und Typ angeben.
	unsigned char MessVar1;    ///< 
	unsigned char MessVar2;    ///< 
	union Ex {
		TBExBestaetigung Bestaetigung; ///< 
		TBExNumFeld NumFeld;           ///< 
		TBExListAuswahl ListAuswahl;   ///< 
	} Ex;

} TBFeldEinlesen;


///////////////////////////////////////
// Struct: StructGlobVar

/** Diese Struktur enthaelt die Namen der definierten Globalen Variablen.

  Diese Struktur zeigt mit P auf den naechsten Eintrag. Sie selbst ist standardmaessig die erste Struktur 
  in den Setupdaten.
*/
typedef struct StructGlobVar {

	char SFeld[8][MAX_NAME]; ///< Namen der GlobVars 
	unsigned int P;         ///< Zeiger auf Root StructBedien 

} SGlobVar;


///////////////////////////////////////
// Struct: StructConfig

// Transpondertypen: TRANSPONDER_TYPE
#define TRANSPONDER_NONE	                 0x3F
#define TRANSPONDER_UNIQUE	                 0
#define TRANSPONDER_HITAG2                   1
#define TRANSPONDER_LEGIC                    2
#define TRANSPONDER_HITAG1                   3
#define TRANSPONDER_HITAGS_H32               4
#define TRANSPONDER_MIFARE                   5
#define TRANSPONDER_Q5                       6
#define TRANSPONDER_HITAGS_H56               7
#define TRANSPONDER_HITAGS_H48               8
#define TRANSPONDER_IBUTTON                  9
#define TRANSPONDER_SIMONSVOSS              10
#define TRANSPONDER_NEDAP                   11
#define TRANSPONDER_TITAN                   12
#define TRANSPONDER_HITAG1_CRYPT            13
#define TRANSPONDER_HITAG1_SECURITY         14
#define TRANSPONDER_HID_PROXKEY             15
#define TRANSPONDER_HID_ICLASS              16
#define TRANSPONDER_LEGIC_ADVANT            17
#define TRANSPONDER_MIFARE_DESFIRE          18
#define TRANSPONDER_MIFARE_PLUS             19
#define TRANSPONDER_MIFARE_ULTRA_LIGHT      20
#define TRANSPONDER_ICODE                   21
#define TRANSPONDER_MYD                     22


/** Diese Struktur enthaelt die Grundeinstellungen zum Geraet.

  Diese Struktur beinhaltet alle Gruneinstellungen des Geraets, sowie globale Einstellungen die ggf. an anderen
  Stellen angepasst werden koennen.
*/
typedef struct StructConfig {

    unsigned char nu01[3];                     ///< (0x00) Busnummer
	                                           ///< (0x02) Frei (war hibyte von LCD_LIGHT)
	unsigned char LCD_LIGHT;                   ///< (0x03) Hintergrundbeleuchtung LCD
	unsigned short ONLINE_WAIT;                ///< (0x04) Wartezeit auf Antwort vom PC
	unsigned char nu02;                        ///< (0x06) Frei (war hibyte von SERVER_ONLINE)
	unsigned char SERVER_ONLINE;               ///< (0x07) Offlinebetrieb
	unsigned short SYSTEMMENU_PASSWORD;        ///< (0x08) Passwort fuer bios
	unsigned char nu03[7];                     ///< (0x0A) MDE_DIVIDER beim AEIII+
	                                           ///< (0x0C) MDE_TIMEOUT beim AEIII+
	                                           ///< (0x0E) MDE_ZYKLUS beim AEIII+
                                               ///< (0x10) Frei (war hibyte von TERMINAL_MODE)
	unsigned char TERMINAL_MODE;               ///< (0x11) normal, PZE1, PZE2, MDE
	unsigned char nu04[11];                    ///< (0x12) Unikatsmodus (-> Es wird die UID der Karte gelesen)
	                                           ///< (0x13) Unterscheidung zwischen segmentiert und unsegmentiert
	                                           ///< (0x14) ACTION_E1_UP beim AEIII+
	                                           ///< (0x16) ACTION_E1_DOWN beim AEIII+
	                                           ///< (0x18) ACTION_E2_UP beim AEIII+
	                                           ///< (0x1A) ACTION_E2_DOWN beim AEIII+
	                                           ///< (0x1C) Frei (war hibyte von ONOFF_DEACTIVATE)
	unsigned char ONOFF_DEACTIVATE;            ///< (0x1D) Ein-/Austaste aktivieren
	unsigned char nu05;                        ///< (0x1E) Frei (war hibyte von INFO_TABLEAU)
	unsigned char INFO_TABLEAU;                ///< (0x1F) Infoseite anzeigen
	unsigned char UNIQUE_Mode;                 ///< (0x20) Darstellungsformat (Dec., Hex. usw.)
	unsigned char UNIQUE;                      ///< (0x21) Modus transponder (4/5 Byte)
	// Frueherer MDE - Bereich fuer Transponder verwenden; SW03.01.32
	TBTransponder TP_GLOBAL;                   ///< (0x22) globale Struktur, wie eine Karte gelesen wird
	TBTransponder TP_ACCESS_CONTROL;           ///< (0x28) globale Struktur, wie bei Zutritt gelesen wird
	unsigned char HITAG_AUTOINC_FLAGS;         ///< (0x2E) Flags fuer Autoinkrement bei Karten schreiben
	unsigned short HITAG_AUTOINC_VALUE;        ///< (0x2F) 16BitWert um den Inkrementiert wird
	unsigned char TRANSPONDER_CUT_FLAGS;       ///< (0x31) Flags fuer Zuschneidefunktion der Legic/Mifare Leser
	unsigned char TRANSPONDER_CUT_FROM;        ///< (0x32) Start fuer Ausschneiden von Transponderwert bei Legic & Mifare
	unsigned char TRANSPONDER_CUT_LENGTH;      ///< (0x33) Laenge des auszuschneidenen Transponderwerts 
	unsigned char nu06;                        ///< (0x34) Frei (war hibyte von IOMODULE_ACTIVE)
	unsigned char IOMODULE_ACTIVE;             ///< (0x35)  1 ist aktiv, 0 ist deaktiviert. 
	unsigned char TRANSPONDER_SEGMENT;         ///< (0x36) Segmentnummer bei segmentierten Karten
	unsigned char TRANSPONDER_TYPE;            ///< (0x37) Transponder Type: 0:Unique, 1:Hightech, 2:Legic (Highbit gibt Segmentverwendung an, naechstes bit fuer -nur Seriennummer lesen-)
	unsigned char nu07;                        ///< (0x38) (war hibyte von KEY_SOUND)
	unsigned char KEY_SOUND;                   ///< (0x39) Tastenton
	unsigned char nu08[5];	                   ///< (0x3A) Display - Kontrast
	                                           ///< (0x3C) ? frei
	                                           ///< (0x3E) Frei (war hibyte
	unsigned char LED_PERMANENT;               ///< (0x3F) Letzte FTaste durch blinken abzeigen, wie AE3
	unsigned char nu09[2];                     ///< (0x40) TimeOut fuer MB
	unsigned int HITAG_INITIAL_VALUE_3;       ///< (0x42) Startwert fuer Kartensegment 3 beschreiben (ist im AEIII+ die IP-Adresse)
	unsigned char ENABLE_ENTRANCE;             ///< (0x46) Zutrittskontolle Version 1 aktivieren
	unsigned char EXT_READER;                  ///< (0x47) externer Leser verwenden
	unsigned char DOOR_SIGNAL_TIME;            ///< (0x48) Dauer fuer Tueroeffner
	unsigned char DOOR_MONITORING;             ///< (0x49) Tuerueberwachung aktivieren
	unsigned char DOOR_ALARM_TIME;             ///< (0x4A) Alarmzeit fuer geoeffnete Tuer
	unsigned char ENTRANCE_MAIN_MENU;		   ///< (0x4B) Bit 0: Ueberpruefen von Zutritt im Hauptmenue
	 
	unsigned char ENABLE_CUT;                  ///< (0x4C) Zuschneiden aktivieren
	unsigned char CUT_AT;                      ///< (0x4D) Zuschnitt von
	unsigned char CUT_TO;                      ///< (0x4E) Zuschnitt bis
	unsigned char nu10[5];                     ///< (0x4F) Kartenpraefix fuer Hitag
	                                           ///< (0x53) Segment in/aus welches dieses geschrieben/gelesen wird
	unsigned int HITAG_INITIAL_VALUE_1;       ///< (0x54) Startwert fuer Kartensegment 1 beschreiben
//	unsigned char nu11;                        ///< (0x58) Typ fuer Wert des Kartenpraefix
	unsigned char ENTRANCE_FLAG;               ///< (0x58) Bit 0: Prfung der Feiertage bei Zeitsteuerung der digitalen Ausgnge
	unsigned char EVER_BIG_WATCH;              ///< (0x59) Grosze Uhr immer anzeigen
	unsigned char TIMEOUT_PZE;                 ///< (0x5A) Timeout nach Eingaben zur Pruefung der Tastenumschalttabelle
	unsigned char MDE_LAMP_FLAGS;              ///< (0x5B) Flags fuer die (rote) Lampensteuerung der MDE
	unsigned char PEEP_FLAGS;                  ///< (0x5C) bit(0): wenn gesetzt, kein Bestaetigungspeep fuer OK; bit(1): wenn gesetzt, kein Bestaetigungspeep fuer ERROR
	                                           ///< (0x5C) War vor 04.01.05: Feste Stellenanzahl fuer Kartenwert; 0 => aus; wird links mit 0en aufgefuellt
	unsigned short TRANSPONDER_USER_PASSWORD;  ///< (0x5D) Passwort fuer Benutzermenue; 0 => aus; 16bit
	unsigned int HITAG_INITIAL_VALUE_2;       ///< (0x5F) Startwert fuer Kartensegment 2 beschreiben
	unsigned char ENABLE_ENTRANCE2;            ///< (0x63) Zutrittskontrolle Version 2 aktivieren

} TConfig;

// Include wird von DLL und Setup gemeinsam verwendet.
#include "ExtendedFlash.h"

#pragma pack()

#endif
