// DllTest.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "stdafx.h"

#include "AE3Com.h"      // Aus Projekt: AESetup
#include "AE3ErrorDef.h" // Aus Projekt: AE3ComProject

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

// Allgemeine, gemeinsam verwendete Information.
struct {

	char szErrorMsg[254];

	unsigned char ucDeviceType;
	unsigned char ucComNum;
	unsigned char ucBusNum;
	char szHost[80];
	int iPortNum;

} g_CommonInfo;


// ################### BENUTZER - EINGABE #################

// Alle Kommandos beginnen mit CMD
#define CMD_NO						-1

// Menuetxte auf Bildschirm ausgeben
void PrintMenu(void)
{

	system("cls");

	#define CMD_END					0
	printf("0  Ende.\n");

	#define CMD_GET_SERIAL_NUM		1
	printf("1  Seriennummer lesen.\n");

	#define CMD_CHECK_AE			2
	printf("2  Testen ob Geraet antwortet.\n");

	#define CMD_SET_DATETIME 		3
	printf("3  Datum/Uhrzeit des AEIII+ mit PC abgleichen.\n");

	#define CMD_GET_DATETIME		4
	printf("4  Datum/Uhrzeit aus AEIII+ lesen und anzeigen.\n");

	#define CMD_LOAD_SETUP			5
	printf("5  Setupdatei in Geraet laden.\n");

	#define CMD_LOAD_LIST			6
	printf("6  Listendaten in AEIII+ laden (nur in Verbindung mit \"AE3pTSetup.aes\")\n");

	#define CMD_SEND_INFO			7
	printf("7  Infotext zum AEIII+ senden.\n");
	
	#define CMD_SEND_MESSAGE		8
	printf("8  Message zum AEIII+ senden.\n");
	
	#define CMD_GET_RD				9 // RecordDescription
	printf("9  Datensatzbeschreibung lesen.\n");

	#define CMD_GET_LD				10 // ListDescription
	printf("10 Listenbeschreibung lesen.\n");
	
	#define CMD_GET_RECORD			11
	printf("11 Daten aus AEIII+ auslesen und Rohdaten anzeigen.\n");

	#define CMD_CLEAR 				12
	printf("12 Daten loeschen.\n");
	
	#define CMD_LOG_ON 				13
	printf("13 Log einschalten. (Datei: C:\\DFComDLL.log)\n");
	
	#define CMD_LOG_OFF				14
	printf("14 Log ausschalten.\n");

	#define CMD_TEST 				15
	printf("15 Testeintrag fuer Programmerweiterung.\n");

	#define CMD_MAX_0				15

	printf("---------------------------------------------------------------\n");

	#define CMD_COM_NUM				30
	printf("30 COM [255=Fehler]    einstellen (aktuell: %i)\n", g_CommonInfo.ucComNum);

	#define CMD_HOST 				31
	printf("31 TCP/IP              einstellen (aktuell: %s)\n", g_CommonInfo.szHost);
	
	#define CMD_HOST_PORT 			32
	printf("32 TCP/IP - Port       einstellen (aktuell: %i)\n", g_CommonInfo.iPortNum);
	
	#define CMD_BUS_NUM				33
	printf("33 Busnummer           einstellen (aktuell: %i)\n", g_CommonInfo.ucBusNum);
	
	#define CMD_DEVICE_TYPE			34
	printf("34 Typ [0=AEIII+,1=PZE Master IV] (aktuell: %i)\n", g_CommonInfo.ucDeviceType);

	#define CMD_MAX_1				34

	printf("---------------------------------------------------------------\n");
	printf("Ergebnis: %s\n", g_CommonInfo.szErrorMsg);
	printf("Eingabe: ");
}

// Menuetexte ausgeben und Kommando ermitteln
int NextCommand(void)	
{
	char Buf[200];
	int iRes;

	PrintMenu();

	memset(Buf, 0, sizeof(Buf));
	gets(Buf);
	// Einfaches Return loescht Ergebniszeile
	if (strlen(Buf) == 0) {
		
		return CMD_NO;
	}

	iRes = atoi(Buf);

	if (((iRes >=  0) && (iRes <= CMD_MAX_0)) || 
		((iRes >= 30) && (iRes <= CMD_MAX_1))) {
	
		return iRes;
	}
	
	return CMD_NO;
}

// ############## KOMMANDO - HILFE ########################

// Beispiel-Listendaten erzeugen (Listenformat ist auf Setup-Datei "TestSetup" abgestimmt)
// Die Listendaten mssen immer exakt mit der Listenbeschreibung im Setup bereinstimmen
// Zeichenketten sind mit Nullen auf die im Setup vorgegebene Feldlnge zu fllen
// 
// Die Liste hat das Format der ersten (und einzigen) Listenbeschreibung im Setup (ListNum -> 0)
// Die Liste besteht aus folgenden 2 Feldern:
//		Personalnummer: Datentyp: unsigned char -> 4 Byte (3 Zeichen + abschliesende 0)
//		Name:			Datentyp: unsigned char -> 17 Byte (16 Zeichen + abschliesende 0)
// es werden 3 Zeilen erzeugt
#define LF_1_LEN	4   // Listenfeld 1 Laenge in Byte
#define LF_2_LEN	17	// Listenfeld 2 Laenge in Byte

#define LZ_LEN		(LF_1_LEN + LF_2_LEN)	// Listenzeilenlaenge in Byte
#define L_ZEILEN	3

void MakeSampleList(char *Buf)
{
	// Beispiel Personalnummern
	char PersNum[L_ZEILEN][LF_1_LEN] = {"101", "102", "103"};	
	// Beispielnamen
	char Namen[L_ZEILEN][LF_2_LEN]   = {"Mueller", "Maier", "Schmidt"};	
	
	// Zeilen erzeugen
	for (int i=0; i < L_ZEILEN; i++) {	

		// Personalnummer in Listenzeile eintragen
		strcpy(&Buf[i * LZ_LEN], PersNum[i]);
		// Name in Listenzeile eintragen
		strcpy(&Buf[i * LZ_LEN + LF_1_LEN], Namen[i]);
	}
}


// #################### AUSGABE ###########################

// Textausgabe auf Konsole
void PrintMsg(const char* pszFormat, ...)
{
	char buf[1024];
	sprintf(buf, "> ");
	va_list arglist;
	va_start(arglist, pszFormat);
	vsprintf(&buf[strlen(buf)], pszFormat, arglist);
	va_end(arglist);
	strcat(buf, "\n");

	if(stdout != NULL)
	{
		fprintf(stdout, buf);
	}
}

// Kopiert ErrorText in Variable der globalen - Struktur.
void PrintError(int iError) 
{
	for (int i=0; i < sizeof(g_CommonInfo.szErrorMsg); i++) {
	
		g_CommonInfo.szErrorMsg[i] = 0;
	}

#if (COM_VERSION >= 0x02000300)

	DFCGetErrorText(g_CommonInfo.ucComNum, iError, 0, g_CommonInfo.szErrorMsg, 254);

#else
	switch ( iError ) {
	case ERROR_CRC:          // CRC Fehler
		strcpy(g_CommonInfo.szErrorMsg, "CRC Fehler.");
		break;
	case ERROR_COM_INIT:     // Oeffnen der Schnittstelle fehlgeschlagen
		strcpy(g_CommonInfo.szErrorMsg, "Oeffnen der Schnittstelle fehlgeschlagen.");
		break;
	case ERROR_COM_CLOSED:   // Schnittstelle ist geschlossen
		strcpy(g_CommonInfo.szErrorMsg, "Schnittstelle ist geschlossen.");
		break;
	case ERROR_COM:          // Allgemeiner Fehler waehrend Kommunikation
		strcpy(g_CommonInfo.szErrorMsg, "Allgemeiner Fehler waehrend Kommunikation.");
		break;
	case ERROR_NO_ANSWER:    // Geraet antwortet nicht
		strcpy(g_CommonInfo.szErrorMsg, "Geraet antwortet nicht.");
		break;
	case ERROR_AT_SENDING:   // Fehler waehrend senden eingetreten
		strcpy(g_CommonInfo.szErrorMsg, "Fehler waehrend senden eingetreten.");
		break;
	case ERROR_AT_RECIEVING: // Fehler waehrend empfang eingetreten
		strcpy(g_CommonInfo.szErrorMsg, "Fehler waehrend empfang eingetreten.");
		break;
	case ERROR_SENDER:       // Antwort von falschem Geraet oder Antwort nicht sinnvoll
		strcpy(g_CommonInfo.szErrorMsg, "Antwort von falschem Geraet oder Antwort nicht sinnvoll.");
		break;
	case ERROR_BUS_NUM:      // Keine gueltige Busnummer (1 - 31, 254)
		strcpy(g_CommonInfo.szErrorMsg, "Keine gueltige Busnummer (1 - 31, 254).");
		break;
	case ERROR_CHANNEL:      // Kanal nicht vorhanden oder nicht erstellbar.
		strcpy(g_CommonInfo.szErrorMsg, "Kanal nicht vorhanden oder nicht erstellbar.");
		break;
	case ERROR_OPEN:         // Oeffnen einer Datei gescheitert
		strcpy(g_CommonInfo.szErrorMsg, "Oeffnen einer Datei gescheitert.");
		break;
	case ERROR_FORMAT:       // z.B. falsches Format einer Setup-Datei
		strcpy(g_CommonInfo.szErrorMsg, "Falsches Format z.B. bei einer Setup-Datei.");
		break;
	case ERROR_LIST:         // Fehler bei Listendatenverarbeitung
		strcpy(g_CommonInfo.szErrorMsg, "Fehler bei Listendatenverarbeitung.");
		break;
	case ERROR_FW_VERSION:   // Firmwareversion nicht kompatible zu Routine
		strcpy(g_CommonInfo.szErrorMsg, "Firmwareversion nicht kompatible.");
		break;
	case ERROR_IS_DATA:		// Es sind noch Daten im Geraet vorhanden.
		strcpy(g_CommonInfo.szErrorMsg, "Es sind noch Daten im Geraet vorhanden.");
		break;
	case ERROR_BREAK:		// Durchfuehrung wurde von Anwender abgebrochen.
		strcpy(g_CommonInfo.szErrorMsg, "Durchfuehrung wurde von Anwender abgebrochen.");
		break;
	case ERROR_NO_RD:		// Keine Datensatzbeschreibung geladen.
		strcpy(g_CommonInfo.szErrorMsg, "Keine Datensatzbeschreibung geladen.");
		break;
	case ERROR_NO_LD:		// Keine Listenbeschreibung geladen.
		strcpy(g_CommonInfo.szErrorMsg, "Keine Listenbeschreibung geladen.");
		break;
	default:
		strcpy(g_CommonInfo.szErrorMsg, "undefinierter Fehler.");
		break;
	}
#endif
}

void PrintDaten(int Anz,unsigned char *Buf, int Len)
{
	int i;

	printf("Nr.:%5d Dat.:", Anz);

	for (i=0; i < Len; i++) {

		printf("%02X ", Buf[i]);
	}

	printf("\n");
}

void PrintRawData(unsigned char *pucBuf, int iLength)
{
	printf("\n");

	for (int i=0; i < iLength; i++) {

		printf("%02X ", pucBuf[i]);
	}

	printf("\n");
}

void PrintDatumUhrzeit(unsigned char *buf)
{
	sprintf(g_CommonInfo.szErrorMsg, "%02d.%02d.%02d%02d %02d:%02d:%02d",
				buf[3],buf[2],buf[0],buf[1],buf[4],buf[5],buf[6]);
}

void PrintDatensatzbeschreibung(int ComNr)
{
	int i, j, cnt, FeldCnt, typ;
	unsigned char name[17];

	for (i=0; i< DFCDatBCnt(ComNr); i++) {

		DFCDatBDatensatz(ComNr, i, name, &FeldCnt);

		printf ("Datensatzbeschreibung-Nr.:%2d \"%s\" Felder:%2d\n",i,name,FeldCnt);
		
		for (j=0; j < FeldCnt; j++) {

			DFCDatBFeld(ComNr, i, j, name, &typ, &cnt);

			printf ("     Feld-Nr.:%2d \"%s\" Typ:%1d  Groesse in Byte:%2d\n", j, name, typ, cnt);
		}
	}

	PrintMsg("Zum weitermachen Enter betaetigen ...");
	getchar();
}

void PrintListenbeschreibung(int ComNr)
{
	int i, j, cnt, FeldCnt, typ, var;
	unsigned char name[17];

	for (i=0; i< DFCListBCnt(ComNr); i++) {

		DFCListBDatensatz(ComNr, i, name, &FeldCnt, &var);

		printf ("Listenbeschreibung-Nr.:%2d \"%s\" Felder:%2d\n", i, name, FeldCnt);

		for (j=0; j < FeldCnt; j++) {

			DFCListBFeld(ComNr, i, j, name, &typ, &cnt);

			printf ("     Feld-Nr.:%2d \"%s\" Typ:%1d  Groesse in Byte:%2d\n", 
				j, name, typ, cnt);
		}
	}

	PrintMsg("Zum weitermachen Enter betaetigen ...");
	getchar();
}

// Verbindung vor Aufruf schliessen!!
int Open()
{
	char szComString[5][5] = {"", "COM1", "COM2", "COM3", "COM4"};
	int iBusNum = 1, // Busnummer der DLL
		iComNum = g_CommonInfo.ucComNum; // Kanalnummer halten
	// Nach einem Fehlschlag bleibt sie ungueltig
	g_CommonInfo.ucComNum = -1;

	// Aktuelle Verbindung schliessen
	if (g_CommonInfo.ucComNum != -1) {
	
		DFCComClose(g_CommonInfo.ucComNum);
	}

	// TCP / IP - Adresse vorhanden?
	if (strcmp(g_CommonInfo.szHost, "0.0.0.0") == 0) {

		// Serielle Verbindung ueber COM4 ist hier nicht moeglich.
		if (iComNum > 4) {
		
			return FALSE;
		}

		// AEIII+ ?
		if (g_CommonInfo.ucDeviceType == 0) {

			if (!DFCComOpenSerial(iComNum, szComString[iComNum], 38400, 300)) {
			
				return FALSE;
			}
		} else {

			if (!DFCComOpenIV(iComNum, iBusNum, 1, szComString[iComNum], 38400, 300)) {
			
				return FALSE;
			}
		}


	} else {
	
		// AEIII+ ?
		if (g_CommonInfo.ucDeviceType == 0) {

			if (!DFCComOpenSocket(iComNum, g_CommonInfo.szHost, g_CommonInfo.iPortNum, 3000)) {
			
				return FALSE;
			}
		} else {

			if (!DFCComOpenIV(iComNum, iBusNum, 3, g_CommonInfo.szHost, g_CommonInfo.iPortNum, 3000)) {
			
				return FALSE;
			}
		}
	}

	// verwendete COM - Nummer zuweisen.
	g_CommonInfo.ucComNum = iComNum;

	return TRUE;
}

// ################ MAIN ###################
// Hautproutine fuer DLL - Testprogramm

int main(int argc, char* argv[])
{
	// Fuer Befehlsausfuehrung
	int iCommand, iError, ret;
	char szBuf[80];

	// Ausfuehren der Hauptschleife
	unsigned char ucEnd = FALSE;
	// Loggen angeschaltet
	unsigned char ucLog = FALSE;

	// Puffer fuer DateTime
	unsigned char pucDTBuf[7];

	// Puffer fr Beispiellistendaten
	unsigned char pucListBuf[0xFF]; 
	
	// Speicher fuer Datensatzdaten
	unsigned char pucDataBuf[0xFF];	
	int iDataLen;

	char InfoText[]="Infotext 1\rInfot. Zeile 2\0";
	
	// Seriennummer lesen
	int iSerialNum;

	// Dateiname fuer Setupdaten
	char pucFileName[0xFF];

	// Kanalnummer, Busnummer und Hostadresse einstellen
	g_CommonInfo.ucComNum = 1;
	g_CommonInfo.ucBusNum = 254;	// Busnummer 254 -> Offline 

	strcpy((char*)g_CommonInfo.szHost, "0.0.0.0");
	g_CommonInfo.iPortNum = 8000;

	// Funktionen zum Initialisieren der Schnittstelle
	if (!DFCComInit( g_CommonInfo.ucComNum )) {
	//if (!DFCComOpen(g_CommonInfo.ucComNum, g_CommonInfo.szHost)) {
	//if (!DFCComOpenIV(g_CommonInfo.ucComNum, 1, 1, szComString[g_CommonInfo.ucComNum], 38400, 300)) {
	//if (!DFCComOpenIV(g_CommonInfo.ucComNum, 1, 1, g_CommonInfo.szHost, g_CommonInfo.iPortNum, 3000)) {

		g_CommonInfo.ucComNum = -1;
		strcpy(g_CommonInfo.szErrorMsg, "Fehler beim Initialisieren des Kanals.");
	}

	// Hautpschleife
	while ( !ucEnd ) {

		// Menue ausgeben und Befehl abfragen
		iCommand = NextCommand();
		memset(g_CommonInfo.szErrorMsg, 0, sizeof(g_CommonInfo.szErrorMsg));

		switch ( iCommand ) {
		case CMD_LOAD_LIST:

			// Puffer mit Beispiellistendaten aufbauen
			MakeSampleList((char*)pucListBuf);	
			
			// Listen-Puffer in DLL loeschen
			DFCClrListenBuffer(g_CommonInfo.ucComNum);		
			// Listen-Puffer in DLL uebertragen
			DFCMakeListe(g_CommonInfo.ucComNum, 0, L_ZEILEN, L_ZEILEN * LZ_LEN, pucListBuf, 0);

			PrintMsg(" Bitte warten ....");

			// Listen-Puffer zum Geraet uebertragen
			if (!DFCLoadListen(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError)) {

				PrintError(iError);
				break;
			}

			strcpy(g_CommonInfo.szErrorMsg, "Listendaten wurden erfogreich geladen.");
			break;

		case CMD_GET_RECORD:

			// Daten abholen
			switch (DFCComCollectData(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError)) {
			case -1: // Lesen gescheitert
				PrintError(iError);
				break;
			case 0: // Keine Datensaetzte vorhanden
				strcpy(g_CommonInfo.szErrorMsg, "Keine Datensaetze vorhanden.");
				break;
			case 1: // Offline Datensatz gelesen
				strcpy(g_CommonInfo.szErrorMsg, "Offline Datensatz gelesen.");
				break;
			case 2: // Online Datensatz gelesen
				strcpy(g_CommonInfo.szErrorMsg, "Online Datensatz gelesen.");
				break;
			}

			do {
				switch (ret = DFCComGetDatensatz(g_CommonInfo.ucComNum, pucDataBuf, &iDataLen, &iError)) {
				case -1:
					PrintError(iError);
					break;
				case 0:
					strcpy(g_CommonInfo.szErrorMsg, "Kein Datensatz vorhanden.");
					break;
				case 1: // Datensatz in Folge.
				case 2: // Letzter Datensatz.
					PrintRawData(pucDataBuf, iDataLen);
					break;
				}
			} while (ret == 1);

			PrintMsg("Zum weitermachen Enter betaetigen ...");
			getchar();
			break;

		case CMD_GET_DATETIME:

			if (!DFCComGetTime(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, pucDTBuf)) {
				
				strcpy(g_CommonInfo.szErrorMsg, "Fehler beim lesen vom Datum/Uhrzeit eingetreten."); 
				break;
			}

			PrintDatumUhrzeit( pucDTBuf ); 
			break;
		
		case CMD_SET_DATETIME:

			time_t tm_time;
			struct tm *ptm;
			tm_time = time(NULL);
			ptm = localtime(&tm_time);
			
			pucDTBuf[0] = (ptm->tm_year + 1900) / 100;
			pucDTBuf[1] = (ptm->tm_year + 1900) % 100;
			pucDTBuf[2] = ptm->tm_mon + 1;
			pucDTBuf[3] = ptm->tm_mday;
			pucDTBuf[4] = ptm->tm_hour;
			pucDTBuf[5] = ptm->tm_min;
			pucDTBuf[6] = ptm->tm_sec;

			if (!DFCComSetTime(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, pucDTBuf)) {
				
				strcpy(g_CommonInfo.szErrorMsg, "Fehler beim setzen vom Datum/Uhrzeit eingetreten."); 
				break;
			}

			PrintDatumUhrzeit( pucDTBuf ); 
			break;

		case CMD_SEND_INFO:

			if (!DFCComSendInfotext(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, InfoText, strlen(InfoText))) {
				
				strcpy(g_CommonInfo.szErrorMsg, "Fehler beim Versenden eingetreten.");
				break;
			}

			strcpy(g_CommonInfo.szErrorMsg, "Infotext erfolgreich gesendet."); 
			break;
		
		case CMD_SEND_MESSAGE:

			if (!DFCComSendMessage(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, 0xFF, 0, 1, InfoText, strlen(InfoText))) {
				
				strcpy(g_CommonInfo.szErrorMsg, "Fehler beim Versenden eingetreten.");
				break;
			}

			strcpy(g_CommonInfo.szErrorMsg, "Messagetext erfolgreich gesendet."); 
			break;
		
		case CMD_LOG_ON:


			// Log - Dateiname setzen
			DFCSetLogFileName(g_CommonInfo.ucComNum, (unsigned char*)"C:\\DFComDLL.log");
			DFCSetLogOn(g_CommonInfo.ucComNum);
			ucLog = TRUE;

			strcpy(g_CommonInfo.szErrorMsg, "Log eingeschaltet."); 
			break;
		
		case CMD_LOG_OFF:

			DFCSetLogOff(g_CommonInfo.ucComNum);
			ucLog = FALSE;

			strcpy(g_CommonInfo.szErrorMsg, "Log ausgeschaltet."); 
			break;

		case CMD_GET_RD:

			PrintMsg(" Bitte warten ....");

			if (!DFCLoadDatensatzbeschreibung(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError)) {

				PrintError(iError);
				break;
			} 

			PrintDatensatzbeschreibung(g_CommonInfo.ucComNum);
			break;

		case CMD_GET_LD:

			PrintMsg(" Bitte warten ....");

			if (!DFCLoadListenbeschreibung(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError)) {

				PrintError(iError);
				break;
			} 

			PrintListenbeschreibung(g_CommonInfo.ucComNum);
			break;

		case CMD_GET_SERIAL_NUM:

			if (!DFCGetSeriennummer(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError, &iSerialNum)) {

				PrintError(iError);
				break;
			} 

			sprintf(g_CommonInfo.szErrorMsg, "Seriennummer ist ( %i )", iSerialNum);
			break;

		case CMD_CHECK_AE:

			if (!DFCCheckDevice(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, &iError, 1)) {

				PrintError(iError);
				break;
			} 

			strcpy(g_CommonInfo.szErrorMsg, "AEIII+ ist erreichbar.");
			break;

		case CMD_LOAD_SETUP:

			printf("Setup - Dateiname eingeben: ");
			memset(pucFileName, 0, sizeof(pucFileName));
			gets(pucFileName); 
			
			if (strlen(pucFileName) < 5) {
			
				// Namenangabe zu kurz
				strcpy(g_CommonInfo.szErrorMsg, "Zu kurze Eingabe.");
				break;
			}

			PrintMsg(" Bitte warten ....");
			
			if (!DFCSetupLaden(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum, pucFileName, &iError)) {

				PrintError(iError);
				break;
			}
			
			strcpy(g_CommonInfo.szErrorMsg, "Setupdatei wurde erfolgreich geladen.");
			break;

		case CMD_CLEAR:

			if (!DFCComClearData(g_CommonInfo.ucComNum, g_CommonInfo.ucBusNum)) {

				strcpy(g_CommonInfo.szErrorMsg, "Daten konnten nicht geloescht werden.");
				break;
			} 

			strcpy(g_CommonInfo.szErrorMsg, "Daten wurden geloescht.");
			break;

		case CMD_COM_NUM:
			
			memset(szBuf, 0, sizeof(szBuf));
			
			printf("Kanalnummer: ");
			
			gets(szBuf);
			iCommand = atoi(szBuf);

			if ((iCommand != g_CommonInfo.ucComNum) && 
				(iCommand >= 1) && (iCommand <= 10)) {
				
				// Offene Verbindung schliessen
				if (g_CommonInfo.ucComNum != -1) {
				
					DFCComClose(g_CommonInfo.ucComNum);
				}
				g_CommonInfo.ucComNum = iCommand;

				// Oeffnen je nach Verwendung
				if (!Open()) {
				
					strcpy(g_CommonInfo.szErrorMsg, "Fehler beim initialisieren des Kanals.");
					break;
				}

				strcpy(g_CommonInfo.szErrorMsg, "Kanal erfolgreich gewechselt.");
			} else {

				strcpy(g_CommonInfo.szErrorMsg, "Kanal wird bereits verwendet oder Bereichsfehler (1 - 10).");
			}

			break;

		case CMD_BUS_NUM:

			memset(szBuf, 0, sizeof(szBuf));
			
			printf("Busnummer: ");
			
			gets(szBuf);
			iCommand = atoi(szBuf);

			if ((iCommand != g_CommonInfo.ucBusNum) && 
				((iCommand >= 1) && (iCommand <= 31) || (iCommand == 254)) ) {
				
				g_CommonInfo.ucBusNum = iCommand;
				strcpy(g_CommonInfo.szErrorMsg, "Busnummer erfolgreich gewechselt.");
			} else {

				strcpy(g_CommonInfo.szErrorMsg, "Busnummer wird bereits verwendet oder Bereichsfehler (1 - 31 / 254).");
			}

			break;

		case CMD_HOST:

			memset(szBuf, 0, sizeof(szBuf));
			
			printf("Hostname oder IP: ");
			
			gets(szBuf);

			if ((strcmp(szBuf, g_CommonInfo.szHost) != 0) ||
				(strcmp(szBuf, "0.0.0.0") == 0) ) {
				
				if (g_CommonInfo.ucComNum != -1) {
				
					DFCComClose(g_CommonInfo.ucComNum);
				}
				strcpy(g_CommonInfo.szHost, szBuf);

				if (!Open()) {

					strcpy(g_CommonInfo.szHost, "0.0.0.0"); // loeschen
					strcpy(g_CommonInfo.szErrorMsg, "Fehler beim initialisieren des Kanals.");
					break;
				}

				strcpy(g_CommonInfo.szErrorMsg, "Kanal erfolgreich gewechselt.");
			} else {

				strcpy(g_CommonInfo.szErrorMsg, "Kanal wird bereits verwendet.");
			}
			break;

		case CMD_HOST_PORT:

			memset(szBuf, 0, sizeof(szBuf));
			
			printf("Portnummer: ");
			
			gets(szBuf);
			iCommand = atoi(szBuf);

			if ((iCommand != g_CommonInfo.iPortNum) && (iCommand >= 1)) {
				
				if (g_CommonInfo.ucComNum != -1) {
				
					DFCComClose(g_CommonInfo.ucComNum);
				}
				g_CommonInfo.iPortNum = iCommand;

				if (!Open()) {

					strcpy(g_CommonInfo.szErrorMsg, "Fehler beim initialisieren des Kanals.");
					break;
				}
				strcpy(g_CommonInfo.szErrorMsg, "Portnummer erfolgreich gewechselt.");
			} else {

				strcpy(g_CommonInfo.szErrorMsg, "Portnummer wird bereits verwendet oder Bereichsfehler ( > 0 ).");
			}
			break;

		case CMD_DEVICE_TYPE:
			
			memset(szBuf, 0, sizeof(szBuf));
			
			printf("Geraetetyp: ");
			
			gets(szBuf);
			iCommand = atoi(szBuf);

			if ((iCommand != g_CommonInfo.ucDeviceType) && 
				(iCommand >= 0) && (iCommand <= 1)) {
				
				if (g_CommonInfo.ucComNum != -1) {
				
					DFCComClose(g_CommonInfo.ucComNum);
				}
				g_CommonInfo.ucDeviceType = iCommand;

				if (!Open()) {

					strcpy(g_CommonInfo.szErrorMsg, "Fehler beim initialisieren des Kanals.");
					break;
				}

				strcpy(g_CommonInfo.szErrorMsg, "Geraetetyp erfolgreich gewechselt.");
			} else {

				strcpy(g_CommonInfo.szErrorMsg, "Geraetetyp wird bereits verwendet oder Bereichsfehler (1 - 10).");
			}

			break;

		case CMD_TEST:
			
			strcpy(g_CommonInfo.szErrorMsg, "Testeintrag fuer Programmerweiterung, noch keine Funktion.");
			break;

		case CMD_END:

			ucEnd = TRUE;
			break;
		}
	}

	if (ucLog) { // Log ausschalten, wenn noch angeschaltet.
	
		DFCSetLogOff(g_CommonInfo.ucComNum);
	}

	// Kanal geoeffnet?
	if (g_CommonInfo.ucComNum != -1) {
		
		DFCComClose(g_CommonInfo.ucComNum);
	}

	return 0;
}
